#include <iostream>
#include <vector>
#include <functional>
#include <cassert>
#include <set>

#define int int64_t

using namespace std;

using graph_t = vector<vector<int>>;

const int ROOT = 0;

const int64_t INF = (1LL << 57);

template <typename T>
vector<T> min_plus(const vector<T> &as, const vector<T> &bs) {
    int n = as.size(), m = bs.size();
    vector<T> res(n + m - 1);
    for (int k = 0; k < n + m - 1; k++) {
        int t = max((int64_t) 0, k - (m - 1));
        res[k] = as[t] + bs[k - t];
        for (int i = 0; i < n; i++) {
            if (k - i < m && k - i >= 0) {
                res[k] = min(res[k], as[i] + bs[k - i]);
            }
        }
    }
    return res;
}

void solve() {
    int n; cin >> n;
    vector<int> parents(n, -1), costs(n);
    graph_t childrens(n);
    for (int i = 1; i < n; i++) {
        cin >> parents[i];
        parents[i]--;
        childrens[parents[i]].push_back(i);
    }
    for (int i = 0; i < n; i++) {
        cin >> costs[i];
    }
    vector<int> ls(n), rs(n);
    for (int i = 0; i < n; i++) {
        cin >> ls[i] >> rs[i];
    }
    set<int> available_vertices;
    vector<int> nums(n), subtree_sums(n);
    int64_t ans = 0;
    bool broken = false;

    function<void(int)> mark_available = [&](int v) {
        if (rs[v] == subtree_sums[v]) {
            return;
        }
        available_vertices.insert(v);
        for (int u : childrens[v]) {
            mark_available(u);
        }
    };
    function<void(int)> dfs = [&](int v) {
        for (int u : childrens[v]) {
            dfs(u);
        }
        if (broken) {
            return;
        }
        available_vertices.clear();
        mark_available(v);
        while (!available_vertices.empty() && subtree_sums[v] < ls[v]) {
            int64_t min_delta = INF;
            int min_u = -1;
            for (int u : available_vertices) {
                if (costs[u] < min_delta) {
                    min_delta = costs[u];
                    min_u = u;
                }
            }
            int needed = ls[v] - subtree_sums[v];
            int max_available = rs[min_u] - subtree_sums[min_u];
            int u = min_u;
            while (u != v) {
                u = parents[u];
                max_available = min(max_available, rs[u] - subtree_sums[u]);
            }
            int added = min(needed, max_available);
            u = min_u;
            subtree_sums[u] += added;
            nums[u] += added;
            ans += added * 1ll * costs[u];
            if (subtree_sums[u] == rs[u]) {
                available_vertices.erase(u);
            }
            while (u != ROOT) {
                u = parents[u];
                if (rs[u] - subtree_sums[u] == added) {
                    available_vertices.erase(u);
                }
                subtree_sums[u] += added;
            }
        }
        if (subtree_sums[v] < ls[v] || subtree_sums[v] > rs[v]) {
            broken = true;
        }
    };
    dfs(ROOT);
    if (ans >= INF || broken) {
        cout << -1 << endl;
        return;
    }
    cout << ans << endl;
    for (int i = 0; i < n; i++) {
        cout << nums[i] << ' ';
    }
    cout << endl;
}

signed main() {
    int t; cin >> t;
    while (t--) {
        solve();
    }
}
