#include <iostream>
#include <vector>
#include <functional>
#include <set>

using namespace std;
#define int long long

const int inf = 1e18;
const pair<int, int> neutral = {inf, inf};

struct Tree {
  vector<pair<int, int>> tree;
  int s;

  Tree(int n) {
    s = 1;
    while (s < n) s *= 2;
    tree.assign(2 * s + 1, neutral);
  }

  void set(int i, pair<int, int> val, int x, int lx, int rx) {
    if (lx + 1 == rx) {
      tree[x] = val;
      return;
    }
    int m = (lx + rx) / 2;
    if (i < m) {
      set(i, val, 2 * x + 1, lx, m);
    } else {
      set(i, val, 2 * x + 2, m, rx);
    }
    tree[x] = min(tree[2 * x + 1], tree[2 * x + 2]);
  }

  pair<int, int> get(int l, int r, int x, int lx, int rx) {
    if (lx >= r || rx <= l) return neutral;
    if (lx >= l && rx <= r) return tree[x];
    int m = (lx + rx) / 2;
    return min(get(l, r, 2 * x + 1, lx, m), get(l, r, 2 * x + 2, m, rx));
  }
};

struct TreeMin {
  vector<int> tree, push;
  int s;

  TreeMin(int n) {
    s = 1;
    while (s < n) s *= 2;
    push.resize(2 * s + 1);
    tree.assign(2 * s + 1, inf);
  }

  void set(int i, int val, int x, int lx, int rx) {
    if (lx + 1 == rx) {
      tree[x] = val;
      return;
    }
    int m = (lx + rx) / 2;
    if (i < m) {
      set(i, val, 2 * x + 1, lx, m);
    } else {
      set(i, val, 2 * x + 2, m, rx);
    }
    tree[x] = min(tree[2 * x + 1], tree[2 * x + 2]);
  }

  void propagate(int x, int lx, int rx) {
    if (lx + 1 == rx || !push[x]) return;
    int p = push[x];
    tree[2 * x + 1] -= p;
    tree[2 * x + 2] -= p;
    push[2 * x + 1] += p;
    push[2 * x + 2] += p;
    push[x] = 0;
  }

  void sub(int l, int r, int val, int x, int lx, int rx) {
    if (lx >= r || rx <= l) return;
    propagate(x, lx, rx);
    if (lx >= l && rx <= r) {
      tree[x] -= val;
      push[x] = val;
      return;
    }
    int m = (lx + rx) / 2;
    sub(l, r, val, 2 * x + 1, lx, m);
    sub(l, r, val, 2 * x + 2, m, rx);
    tree[x] = min(tree[2 * x + 1], tree[2 * x + 2]);
  }

  int get(int l, int r, int x, int lx, int rx) {
    if (lx >= r || rx <= l) return inf;
    propagate(x, lx, rx);
    if (lx >= l && rx <= r) return tree[x];
    int m = (lx + rx) / 2;
    return min(get(l, r, 2 * x + 1, lx, m), get(l, r, 2 * x + 2, m, rx));
  }
};

void solve() {
  int n;
  cin >> n;

  vector<int> p(n, -1);
  vector<vector<int>> g(n);
  for (int i = 1; i < n; i++) {
    cin >> p[i];
    --p[i];
    g[p[i]].push_back(i);
  }
  vector<int> c(n);
  for (auto &x : c) cin >> x;
  vector<int> l(n), r(n);
  for (int i = 0; i < n; i++) {
    cin >> l[i] >> r[i];
  }

  vector<int> size(n), head(n);
  function<void(int)> reorder = [&] (int v) {
    size[v] = 1;
    for (auto &u : g[v]) {
      reorder(u);
      size[v] += size[u];
      if (size[u] > size[g[v][0]]) {
        swap(u, g[v][0]);
      }
    }
  };

  reorder(0);

  int timer = 0;
  vector<int> tin(n), tout(n);
  function<void(int)> hld = [&] (int v) {
    tin[v] = timer++;
    for (auto &u : g[v]) {
      if (u == g[v][0]) {
        head[u] = head[v];
      } else {
        head[u] = u;
      }
      hld(u);
    }
    tout[v] = timer;
  };

  hld(0);

  TreeMin st1(n);

  for (int i = 0; i < n; i++) {
    st1.set(tin[i], r[i], 0, 0, st1.s);
  }

  vector<int> here(n);
  auto upd = [&] (int v, int x) {
    here[v] += x;
    while (v != -1) {
      st1.sub(tin[head[v]], tin[v] + 1, x, 0, 0, st1.s);
      v = p[head[v]];
    }
  };

  auto get_can = [&] (int v) {
//    cout << "can: " << v << ' ';
    int res = inf;
    while (v != -1) {
      res = min(res, st1.get(tin[head[v]], tin[v] + 1, 0, 0, st1.s));
      v = p[head[v]];
    }
//    cout << res << '\n';
    return res;
  };

  Tree st(n);
  bool ok = 1;
  int res = 0;
  function<void(int)> dfs = [&] (int v) {
    if (!ok) return;
    st.set(tin[v], make_pair(c[v], v), 0, 0, st.s);
    for (auto &u : g[v]) {
      dfs(u);
      if (!ok) return;
    }
    while (r[v] - st1.get(tin[v], tin[v] + 1, 0, 0, st.s) < l[v]) {
      int need = l[v] - (r[v] - st1.get(tin[v], tin[v] + 1, 0, 0, st1.s));
//      cout << "need: " << v << ' ' << need << '\n';
      auto can = st.get(tin[v], tout[v], 0, 0, st.s);
//      cout << "need v: " << v << ' ' << need << ' ' << can.first << ' ' << can.second << '\n';
      if (can == neutral) {
        ok = 0;
        return;
      }
      int u = can.second;
      int can_u = get_can(u);
//      cout << "can_u: " << can_u << '\n';
      if (can_u >= need) {
        can_u -= need;
        res += need * c[u];
        upd(u, need);
      } else {
        res += can_u * c[u];
        upd(u, can_u);
        can_u = 0;
      }
      if (can_u == 0) {
        st.set(tin[u], neutral, 0, 0, st.s);
      }
    }
//    cout << "v: " << v << ' ' << here[v] << ' ' << res << '\n';
  };
  dfs(0);

  if (!ok) {
    cout << -1 << '\n';
  } else {
    cout << res << '\n';
    for (int i = 0; i < n; i++) {
      cout << here[i] << ' ';
    }
    cout << '\n';
  }
}

signed main() {
  ios_base::sync_with_stdio(0);
  cin.tie(0);
  int tc;
  cin >> tc;
  while (tc --> 0) solve();
  return 0;
}
