#include <cassert>
#include <algorithm>
#include <iostream>
#include <vector>
#include <set>

using namespace std;

typedef long long ll;

#define all(a) (a).begin(), (a).end()


vector < set < pair < int, int > > > mm;


struct Node {
	int add, mn;

	Node(int x = 0) {
		mn = x;
		add = 0;
	}

	void unite(Node l, Node r) {
		mn = min(l.mn + l.add, r.mn + r.add);
	}
};


class SegmentTree {
	int sz;
	vector < Node > t;

	void build(int v, int l, int r, vector < int >& a) {
		if (l == r) {
			t[v] = Node(a[l]);
			return;
		}

		int md = (l + r) / 2;
		build(2 * v, l, md, a);
		build(2 * v + 1, md + 1, r, a);

		t[v].unite(t[2 * v], t[2 * v + 1]);
	}

	int get_min(int v, int l, int r, int i, int j) {
		if (l == i && r == j)
			return t[v].mn + t[v].add;

		int md = (l + r) / 2;
		if (j <= md) {
			return get_min(2 * v, l, md, i, j) + t[v].add;
		}
		else if (i > md) {
			return get_min(2 * v + 1, md + 1, r, i, j) + t[v].add;
		}
		else {
			return min(get_min(2 * v, l, md, i, md), get_min(2 * v + 1, md + 1, r, md + 1, j)) + t[v].add;
		}
	}

	void change(int v, int l, int r, int i, int j, int add) {
		if (l == i && r == j) {
			t[v].add += add;
			return;
		}

		int md = (l + r) / 2;
		if (j <= md) {
			change(2 * v, l, md, i, j, add);
		}
		else if (i > md) {
			change(2 * v + 1, md + 1, r, i, j, add);
		}
		else {
			change(2 * v, l, md, i, md, add);
			change(2 * v + 1, md + 1, r, md + 1, j, add);
		}

		t[v].unite(t[2 * v], t[2 * v + 1]);
	}

public:
	SegmentTree(vector < int > a) {
		sz = a.size();
		t.resize(4 * sz);
		build(1, 0, sz - 1, a);
	}

	int get_min(int i, int j) {
		if (i > j)
			swap(i, j);
		return get_min(1, 0, sz - 1, i, j);
	}

	void change(int i, int j, int add) {
		if (i > j)
			swap(i, j);
		change(1, 0, sz - 1, i, j, add);
	}
};


void set_ord(int v, vector < vector < int > >& g, vector < int >& sz) {
	sz[v] = 1;
	for (int to : g[v]) {
		set_ord(to, g, sz);
		sz[v] += sz[to];
	}

	sort(all(g[v]), [&](int a, int b) {
		return sz[a] > sz[b];
	});
}


void precalc(int v, vector < vector < int > >& g, vector < int >& tin, int& timer, vector < int >& up) {
	tin[v] = timer++;
	for (int i = 0; i < g[v].size(); i++) {
		if (i == 0)
			up[g[v][i]] = up[v];
		else
			up[g[v][i]] = g[v][i];
		
		precalc(g[v][i], g, tin, timer, up);
	}
}


void sol(int v, vector < vector < int > >& g, vector < int >& p, vector < int >& l, vector < int >& r, vector < int >& c, 
	vector < int >& id, ll& sm, vector < int >& ans, vector < int >& tin, vector < int >& up, SegmentTree& st) {

	int mx = -1;
	for (int to : g[v]) {
		sol(to, g, p, l, r, c, id, sm, ans, tin, up, st);

		if (mx == -1 || mm[mx].size() < mm[id[to]].size())
			mx = id[to];
	}

	if (mx == -1) {
		mm.push_back(set < pair < int, int > >());
		mx = mm.size() - 1;
	}
	id[v] = mx;

	mm[mx].insert({ c[v], v });
	for (int to : g[v]) {
		if (id[to] == mx)
			continue;

		for (auto el : mm[id[to]])
			mm[mx].insert(el);
	}

	vector < pair < int, int > > to_errase;
	for (auto el : mm[mx]) {
		int can_v = st.get_min(tin[v], tin[v]);
		if (r[v] - can_v >= l[v])
			break;

		int cnt = 1e9;
		for (int u = el.second; u >= 0; u = p[up[u]])
			cnt = min(cnt, st.get_min(tin[u], tin[up[u]]));

		if (r[v] - can_v + cnt <= l[v])
			to_errase.push_back(el);
		else
			cnt = can_v + l[v] - r[v];
		sm += (ll)cnt * el.first;
		ans[el.second] += cnt;

		for (int u = el.second; u >= 0; u = p[up[u]])
			st.change(tin[u], tin[up[u]], -cnt);
	}

	for (auto el : to_errase)
		mm[mx].erase(el);
}


signed main() {
	ios_base::sync_with_stdio(0); cin.tie(0); cout.tie(0);

	int t;
	cin >> t;

	for (; t; t--) {
		mm.clear();

		int n;
		cin >> n;

		vector < int > p(n, -1);
		vector < vector < int > > g(n);
		for (int i = 0; i < n - 1; i++) {
			cin >> p[i + 1];

			p[i + 1]--;
			g[p[i + 1]].push_back(i + 1);
		}

		vector < int > c(n), l(n), r(n);
		for (int i = 0; i < n; i++)
			cin >> c[i];

		int timer = 0;
		vector < int > sz(n), tin(n), up(n, 0);
		set_ord(0, g, sz);
		precalc(0, g, tin, timer, up);

		vector < int > can(n);
		for (int i = 0; i < n; i++) {
			cin >> l[i] >> r[i];

			can[tin[i]] = r[i];
		}
		SegmentTree st(can);

		ll sm = 0;
		vector < int > id(n, -1), ans(n, 0);
		sol(0, g, p, l, r, c, id, sm, ans, tin, up, st);

		bool res = true;
		for (int i = 0; i < n; i++) {
			int can_i = st.get_min(tin[i], tin[i]);
			if (can_i < 0 || r[i] - can_i < l[i])
				res = false;
		}

		if (!res) {
			cout << "-1\n";
			continue;
		}

		cout << sm << "\n";
		for (int el : ans)
			cout << el << " ";
		cout << "\n";
	}

	return 0;
}