#include <cassert>
#include <algorithm>
#include <iostream>
#include <vector>
#include <set>

using namespace std;

typedef long long ll;

#define all(a) (a).begin(), (a).end()


struct NodeMST {
	int sm;

	NodeMST(int x = 0) {
		sm = x;
	}

	void unite(NodeMST l, NodeMST r) {
		sm = l.sm + r.sm;
	}
};


class MergeSortTree {
	int sz;
	vector < NodeMST > t;

	int find_pos(int v, int l, int r, int cnt) {
		if (l == r) {
			cnt -= t[v].sm;
			if (cnt < 0)
				return -1;
			return l;
		}

		int md = (l + r) / 2;
		if (t[2 * v].sm >= cnt)
			return find_pos(2 * v, l, md, cnt);
		return find_pos(2 * v + 1, md + 1, r, cnt - t[2 * v].sm);
	}

	void change(int v, int l, int r, int i, int x) {
		if (l == r) {
			t[v] = NodeMST(x);
			return;
		}

		int md = (l + r) / 2;
		if (i <= md)
			change(2 * v, l, md, i, x);
		else
			change(2 * v + 1, md + 1, r, i, x);
		t[v].unite(t[2 * v], t[2 * v + 1]);
	}

	int get_sum(int v, int l, int r, int i, int j) {
		if (l == i && r == j)
			return t[v].sm;

		int md = (l + r) / 2;
		if (j <= md)
			return get_sum(2 * v, l, md, i, j);
		if (i > md)
			return get_sum(2 * v + 1, md + 1, r, i, j);
		return get_sum(2 * v, l, md, i, md) + get_sum(2 * v + 1, md + 1, r, md + 1, j);
	}

public:
	MergeSortTree(int n) {
		sz = n;
		t.resize(4 * n);
	}

	int find_pos(int cnt) {
		return find_pos(1, 0, sz - 1, cnt);
	}

	int get_sum(int i, int j) {
		return get_sum(1, 0, sz - 1, i, j);
	}

	void change(int i, int x) {
		change(1, 0, sz - 1, i, x);
	}
}; 


struct Node {
	int mn;

	Node(int x = 0) {
		mn = x;
	}

	void unite(Node l, Node r) {
		mn = min(l.mn, r.mn);
	}
};


class SegmentTree {
	int sz;
	vector < Node > t;

	void build(int v, int l, int r, vector < int >& a) {
		if (l == r) {
			t[v] = Node(a[l]);
			return;
		}

		int md = (l + r) / 2;
		build(2 * v, l, md, a);
		build(2 * v + 1, md + 1, r, a);

		t[v].unite(t[2 * v], t[2 * v + 1]);
	}

	int get_min(int v, int l, int r, int i, int j) {
		if (l == i && r == j)
			return t[v].mn;

		int md = (l + r) / 2;
		if (j <= md)
			return get_min(2 * v, l, md, i, j);
		if (i > md)
			return get_min(2 * v + 1, md + 1, r, i, j);
		return min(get_min(2 * v, l, md, i, md), get_min(2 * v + 1, md + 1, r, md + 1, j));
	}

public:
	SegmentTree(vector < int > a) {
		sz = a.size();
		t.resize(4 * sz);
		build(1, 0, sz - 1, a);
	}

	int get_min(int i, int j) {
		return get_min(1, 0, sz - 1, i, j);
	}
};


struct Request {
	int r1, c1, r2, c2, i, v1;
};


signed main() {
	ios_base::sync_with_stdio(0); cin.tie(0); cout.tie(0);

	int n;
	cin >> n;

	vector < pair < int, int > > sa;
	vector < int > a(n);
	for (int i = 0; i < n; i++) {
		cin >> a[i];

		a[i]--;
		sa.push_back({ a[i], i });
	}
	sort(all(sa));

	int q;
	cin >> q;

	vector < Request > rq(q);
	vector < pair < int, int > > cc1;
	for (Request& cur : rq) {
		cin >> cur.r1 >> cur.c1 >> cur.r2 >> cur.c2;

		cc1.push_back({ cur.r1, cur.r1 - cur.c1 });
		cc1.push_back({ cur.r2, cur.r2 - cur.c2 });
	}
	sort(all(cc1));

	int id = 0;
	vector < int > vv1;
	MergeSortTree st1(n);
	for (pair < int, int > el : cc1) {
		for (; id < n && sa[id].first < el.first; id++)
			st1.change(sa[id].second, 1);
		vv1.push_back(st1.find_pos(el.second));
	}

	SegmentTree mt(a);
	vector < vector < int > > cc2(n);
	for (Request& cur : rq) {
		int r1 = cur.r1, c1 = cur.c1, r2 = cur.r2, c2 = cur.c2;

		int v1 = vv1[lower_bound(all(cc1), make_pair(r1, r1 - c1)) - cc1.begin()];
		int v2 = vv1[lower_bound(all(cc1), make_pair(r2, r2 - c2)) - cc1.begin()];
		if (v1 > v2) {
			swap(cur.r1, cur.r2);
			swap(cur.c1, cur.c2);
		}
		if (v1 > v2) {
			swap(r1, r2);
			swap(c1, c2);
			swap(v1, v2);
		}
		cur.v1 = v1;

		int i = n;
		if (v1 < v2)
			i = mt.get_min(v1 + 1, v2);
		cur.i = i;

		if (v1 >= 0)
			cc2[v1].push_back(i);
	}

	MergeSortTree st2(n);
	vector < vector < int > > vv2(n);
	for (int i = 0; i < n; i++) {
		st2.change(a[i], 1);

		sort(all(cc2[i]));
		for (int el : cc2[i]) {
			int cnt = 0;
			if (el > 0)
				cnt = st2.get_sum(0, el - 1);
			vv2[i].push_back(cnt);
		}
	}

	for (Request& cur : rq) {
		int r1 = cur.r1, c1 = cur.c1, r2 = cur.r2, c2 = cur.c2;

		int i = cur.i, v1 = cur.v1;

		if (i > min(r1, r2)) {
			if (r1 <= r2)
				cout << r1 << " " << c1 << "\n";
			else
				cout << r2 << " " << c2 << "\n";
			continue;
		}

		if (v1 >= 0)
			cout << i << " " << i - vv2[v1][lower_bound(all(cc2[v1]), i) - cc2[v1].begin()] << "\n";
		else
			cout << i << " " << i << "\n";
	}

	return 0;
}