#include <cassert>
#include <algorithm>
#include <iostream>
#include <vector>
#include <set>

using namespace std;

typedef long long ll;

#define all(a) (a).begin(), (a).end()


struct NodeMST {
	vector < int > v;

	NodeMST(int x = 0) {
		v = { x };
	}

	void unite(NodeMST l, NodeMST r) {
		v.clear();
		for (int i = 0, j = 0; i < l.v.size() || j < r.v.size(); ) {
			if (j == r.v.size() || i < l.v.size() && l.v[i] <= r.v[j]) {
				v.push_back(l.v[i]);
				i++;
			}
			else {
				v.push_back(r.v[j]);
				j++;
			}
		}
	}
};


class MergeSortTree {
	int sz;
	vector < NodeMST > t;

	void build(int v, int l, int r, vector < int >& a) {
		if (l == r) {
			t[v] = NodeMST(a[l]);
			return;
		}

		int md = (l + r) / 2;
		build(2 * v, l, md, a);
		build(2 * v + 1, md + 1, r, a);

		t[v].unite(t[2 * v], t[2 * v + 1]);
	}

	int get_cnt(int v, int l, int r, int i, int j, int x) {
		if (l == i && r == j) {
			int cnt = lower_bound(all(t[v].v), x) - t[v].v.begin();
			return cnt;
		}

		int md = (l + r) / 2;
		if (j <= md)
			return get_cnt(2 * v, l, md, i, j, x);
		if (i > md)
			return get_cnt(2 * v + 1, md + 1, r, i, j, x);
		return get_cnt(2 * v, l, md, i, md, x) + get_cnt(2 * v + 1, md + 1, r, md + 1, j, x);
	}

	int find_pos(int v, int l, int r, int x, int cnt) {
		if (l == r) {
			if (t[v].v[0] < x)
				cnt--;
			if (cnt < 0)
				return -1;
			return l;
		}

		int md = (l + r) / 2;
		int cnt_l = lower_bound(all(t[2 * v].v), x) - t[2 * v].v.begin();
		if (cnt_l >= cnt)
			return find_pos(2 * v, l, md, x, cnt);
		return find_pos(2 * v + 1, md + 1, r, x, cnt - cnt_l);
	}

public:
	MergeSortTree(vector < int > a) {
		sz = a.size();
		t.resize(4 * sz);
		build(1, 0, sz - 1, a);
	}

	int find_pos(int x, int cnt) {
		return find_pos(1, 0, sz - 1, x, cnt);
	}

	int get_cnt(int i, int j, int x) {
		if (i > j)
			return 0;
		return get_cnt(1, 0, sz - 1, i, j, x);
	}
};


struct Node {
	int mn;

	Node(int x = 0) {
		mn = x;
	}

	void unite(Node l, Node r) {
		mn = min(l.mn, r.mn);
	}
};


class SegmentTree {
	int sz;
	vector < Node > t;

	void build(int v, int l, int r, vector < int >& a) {
		if (l == r) {
			t[v] = Node(a[l]);
			return;
		}

		int md = (l + r) / 2;
		build(2 * v, l, md, a);
		build(2 * v + 1, md + 1, r, a);

		t[v].unite(t[2 * v], t[2 * v + 1]);
	}

	int get_min(int v, int l, int r, int i, int j) {
		if (l == i && r == j)
			return t[v].mn;

		int md = (l + r) / 2;
		if (j <= md)
			return get_min(2 * v, l, md, i, j);
		if (i > md)
			return get_min(2 * v + 1, md + 1, r, i, j);
		return min(get_min(2 * v, l, md, i, md), get_min(2 * v + 1, md + 1, r, md + 1, j));
	}

public:
	SegmentTree(vector < int > a) {
		sz = a.size();
		t.resize(4 * sz);
		build(1, 0, sz - 1, a);
	}

	int get_min(int i, int j) {
		return get_min(1, 0, sz - 1, i, j);
	}
};


signed main() {
	ios_base::sync_with_stdio(0); cin.tie(0); cout.tie(0);

	int n;
	cin >> n;

	vector < int > a(n);
	for (int i = 0; i < n; i++) {
		cin >> a[i];

		a[i]--;
	}

	int q;
	cin >> q;

	SegmentTree mt(a);
	MergeSortTree mst(a);
	for (; q; q--) {
		int r1, c1, r2, c2;
		cin >> r1 >> c1 >> r2 >> c2;

		int v1 = mst.find_pos(r1, r1 - c1), v2 = mst.find_pos(r2, r2 - c2);
		if (v1 > v2) {
			swap(r1, r2);
			swap(c1, c2);
			swap(v1, v2);
		}

		int i = n;
		if (v1 < v2)
			i = mt.get_min(v1 + 1, v2);

		if (i > min(r1, r2)) {
			if (r1 <= r2)
				cout << r1 << " " << c1 << "\n";
			else
				cout << r2 << " " << c2 << "\n";
			continue;
		}

		cout << i << " " << i - mst.get_cnt(0, v1, i) << "\n";
	}

	return 0;
}