#include <iostream>
#include <vector>
#include <climits>
#include <map>
#include <algorithm>
std::vector<std::vector<int>> children;
std::vector<int> parent, cost_here, lcons, rcons;
std::vector<int64_t> ansv;
std::vector<int> bought, tbought;
int64_t gans = 0;
bool FAIL = false;
struct cmp
{
	bool operator()(int a, int b) const
	{
		return std::tie(cost_here[a], a) < std::tie(cost_here[b], b);
	}
};
std::pair<int,int> min_cost(int at)
{
	if(tbought[at] >= rcons[at]) return { INT_MAX, -1 } ; // can't buy more
	int c = cost_here[at], i = at;
	for(int to : children[at])
	{
		auto[c2,i2] = min_cost(to);
		if(c2 < c) c = c2, i = i2;
	}
	return {c, i};
}
bool dfs2(int at, int tgt)
{
	if(at == tgt) { tbought[at]++; return 1; }
	for(int to : children[at])
	{
		if(dfs2(to, tgt)) { tbought[at]++; return 1; }
	}
	return 0;
}

bool buy1(int at)
{
	// dfs over subtree, find min cost
	auto[cost, from] = min_cost(at);
	if(from == -1) return 0;
	bought[from]+=1;
	gans += cost_here[from];
	// increase tbought on the path to that vertex
	dfs2(0, from);
	//printf("after buying at %d: ", at); for(int x : tbought) printf("%d ", x); printf("\n");
	return 1;
}
void dfs(int at)
{
	int n_extra = rcons[at], n_rem = lcons[at];
	int64_t ans = 0;
	for(int to : children[at])
	{
		dfs(to);
		if(FAIL) return;
		ans += ansv[to];
		n_rem -= lcons[to];
	}
	if(tbought[at] > rcons[at]) { FAIL = true; return; }
	// buy as cheaply as possible from the subtree
	// recalculate constraints (compute `bought` for subtrees, if it's ==rcons skip the subtree)
	while(n_rem > 0)
	{
		if(!buy1(at)) { FAIL = true; return; }
		n_rem--;
	}
	ansv[at] = ans;
}
int main()
{
	// minimize total cost if l <= sum in each subtree <= r
	// solution exists iff buying minimally to satisfy l in each vertex won't violate any r constraints
	// buying higher in the tree is always acceptable, if it's cheaper and won't violate child constraints
	// buy as little as possible in leaves
	// when merging trees, if there is not enough, buy from the cheapest vertices in the subtree
	// return counter maps (cost, n_remaining), add them for each child
	int T; std::cin >> T;
	while(T --> 0)
	{
		int n; std::cin >> n;
		FAIL = false;
		children.assign(n, {});
		parent.assign(n, {});
		cost_here.assign(n, {});
		lcons.assign(n, {});
		rcons.assign(n, {});
		ansv.assign(n, {});
		bought.assign(n, {});
		tbought.assign(n, {});
		parent[0] = -1;
		for(int i = 0; i < n-1; i++)
		{
			int p; scanf("%d", &p); p--;
			parent[i+1] = p;
			children[p].push_back(i+1);
		}
		for(int i = 0; i < n; i++) scanf("%d", &cost_here[i]);
		for(int i = 0; i < n; i++) scanf("%d%d", &lcons[i], &rcons[i]);
		dfs(0);
		if(!FAIL)
		{
			printf("%lld\n", gans);
			for(int x : bought) printf("%d ", x);
			printf("\n");
		}
		else printf("-1\n");
	}
}
