#include <cassert>
#include <iostream>
#include <algorithm>
#include <vector>
#include <string>
#include <iomanip>
#include <set>
#include <map>
#include <unordered_map>
#include <unordered_set>
#include <queue>
#include <numeric>
#include <cmath>
#include <deque>
#include <sstream>
#include <bitset>
#include <ctime>
#include <random>
#include <functional>

using namespace std;
#define int long long
using ll = long long;
using ld = long double;
constexpr int inf = 1e9 + 7;
constexpr double eps = 1e-6;

pair<int, int> DFS3(int v, const vector<vector<int>> &children, vector<int> &costs, vector<int> &L, vector<int> &res) {
    int s_min = 0, best = v;
    for (int u : children[v]) {
        auto [s_min_, best_] = DFS3(u, children, costs, L, res);
        s_min += s_min_;
        if (costs[best_] < costs[best]) {
            best = best_;
        }
    }

    if (s_min < L[v]) {
        res[best] += L[v] - s_min;
        s_min = L[v];
    }
    return {s_min, best};
}

void Solve3(istream &in, ostream &out) {
    int t;
    in >> t;
    while (t--) {
        int n;
        in >> n;
        vector<vector<int>> children(n);
        for (int v = 1; v < n; ++v) {
            int p;
            in >> p;
            --p;
            children[p].push_back(v);
        }
        vector<int> costs(n);
        for (int &c_i : costs) {
            in >> c_i;
        }
        vector<int> L(n), R(n);
        for (int v = 0; v < n; ++v) {
            in >> L[v] >> R[v];
        }
        vector<int> res(n, 0);
        DFS3(0, children, costs, L, res);
        ll s_cost = 0;
        for (int v = 0; v < n; ++v) {
            s_cost += static_cast<ll>(res[v]) * costs[v];
        }
        out << s_cost << '\n';
//        for (int res_i : res) {
//            out << res_i << ' ';
//        }
//        out << '\n';
    }
}

int DFS(int v, const vector<vector<int>> &children, vector<int> &costs, vector<int> &L, vector<int> &X) {
    int s_min = 0;
    for (int u : children[v]) {
        s_min += DFS(u, children, costs, L, X);
    }
    X[v] = max(static_cast<int>(0), L[v] - s_min);
    return max(L[v], s_min);
}

void S_DFS(int v, auto &children, auto &X, auto &S) {
    S[v] = X[v];
    for (int u : children[v]) {
        S_DFS(u, children, X, S);
        S[v] += S[u];
    }
}


void Solve4(istream &in, ostream &out) {
    int t;
    in >> t;
    while (t--) {
        int n;
        in >> n;
        vector<vector<int>> children(n);
        vector<int> parent(n, -1);
        for (int v = 1; v < n; ++v) {
            in >> parent[v];
            --parent[v];
            children[parent[v]].push_back(v);
        }
        vector<int> costs(n);
        for (int &c_i : costs) {
            in >> c_i;
        }
        vector<int> L(n), R(n);
        for (int v = 0; v < n; ++v) {
            in >> L[v] >> R[v];
        }
        vector<int> X(n, 0);
        DFS(0, children, costs, L, X);

        vector<int> S(n, 0);
        S_DFS(0, children, X, S);

        bool OK = true;
        for (int v = 0; v < n; ++v) {
            if (S[v] > R[v]) {
                OK = false;
            }
        }

        if (!OK) {
            out << "-1\n";
            continue;
        }

        vector<int> order(n);
        iota(order.begin(), order.end(), 0);
        sort(order.begin(), order.end(),
             [&costs](int a, int b) { return costs[a] < costs[b]; });

        vector<int> res(n, 0);
        for (int v : order) {
            S_DFS(0, children, X, S);
            int border = static_cast<ll>(inf) * inf;
            for (int u = v; u != -1 && border >= 0; u = parent[u]) {
                int delta = min(X[u], border);
                X[u] -= delta;
                res[v] += delta;
                border -= delta;
                border = min(border, R[u] - S[u]);
            }
        }

        for (int x_i : X) {
            assert(x_i == 0);
        }
        S_DFS(0, children, X, S);
        for (int s_i : S) {
            assert(s_i == 0);
        }

        ll s_cost = 0;
        for (int v = 0; v < n; ++v) {
            s_cost += static_cast<ll>(res[v]) * costs[v];
        }
        out << s_cost << '\n';
        for (int res_i : res) {
            out << res_i << ' ';
        }
        out << '\n';
    }
}


const int MAXR = 200;
const ll LINF = (ll)inf * inf;


void DFS_stupid(int v, auto & children, auto &L, auto &R, auto &dp, auto &costs) {
    dp[v] = vector<int>(MAXR, LINF);
    for (int i = 0; i < MAXR; ++i) {
        dp[v][i] = costs[v] * i;
    }
    for (int u : children[v]) {
        DFS_stupid(u, children, L, R, dp, costs);
        for (int i = MAXR - 1; i >= 0; --i) {
            int next_val = LINF;
            for (int j = 0; j <= i; ++j) {
                next_val = min(next_val, dp[u][j] + dp[v][i - j]);
            }
            dp[v][i] = next_val;
        }
    }
    for (int i = 0; i < MAXR; ++i) {
        if (i < L[v] || i > R[v]) {
            dp[v][i] = LINF;
        }
    }
}

void SolveStupid(istream &in, ostream &out) {
    int t;
    in >> t;
    while (t--) {
        int n;
        in >> n;
        vector<vector<int>> children(n);
        vector<int> parent(n, -1);
        for (int v = 1; v < n; ++v) {
            in >> parent[v];
            --parent[v];
            children[parent[v]].push_back(v);
        }
        vector<int> costs(n);
        for (int &c_i : costs) {
            in >> c_i;
        }
        vector<int> L(n), R(n);
        for (int v = 0; v < n; ++v) {
            in >> L[v] >> R[v];
        }

        vector<vector<int>> dp(n, vector<int>(MAXR));
        DFS_stupid(0, children, L, R, dp, costs);

        int best = LINF;
        for (int i = 0; i < MAXR; ++i) {
            best = min(best, dp[0][i]);
        }
        out << (best == LINF ? -1 : best) << '\n';
    }
}

using Solution = function<void (istream &, ostream &)>;
using TestGenerator = function<void (ostream &)>;

void StressTest(Solution smart, Solution stupid, TestGenerator generator, ostream &result_out = cout) {
    static constexpr int N_EPOCHS = 100000;
    for (int i = 0; i < N_EPOCHS; ++i) {
        ostringstream test;
        generator(test);
        istringstream in_smart(test.str()), in_stupid(test.str());
        ostringstream out_smart, out_stupid;
        smart(in_smart, out_smart);
        stupid(in_stupid, out_stupid);
        if(out_smart.str() != out_stupid.str()) {
            result_out << "Test:\n";
            result_out << test.str() << '\n';
            result_out << "Smart:\n";
            result_out << out_smart.str() << '\n';
            result_out << "Stupid\n";
            result_out << out_stupid.str() << '\n';
            return;
        }
    }
    result_out << "OK\n";
}


void GenTest(ostream &out) {
    mt19937 rnd(time(0) + 239);
//    int t = rnd() % 10 + 1;
    int t = 1;
    out << t << '\n';
    while (t--) {
        int n = rnd() % 100 + 1;
        out << n << '\n';
        for (int i = 1; i < n; ++i) {
            out << rnd() % i + 1 << ' ';
        }
        out << '\n';
        for (int i = 0; i < n; ++i) {
            out << rnd() % 100 + 1 << ' ';
        }
        out << '\n';
        for (int i = 0; i < n; ++i) {
            out << rnd() % 5 << ' ' << 1000 << '\n';
        }
    }
}

void GenTest2(ostream &out) {
    mt19937 rnd(time(0) + 239);
    out << "1\n";
    int n = rnd() % 100 + 5;
    out << n << '\n';
    for (int i = 1; i < n; ++i) {
        out << rnd() % i + 1 << ' ';
    }
    out << '\n';
    for (int i = 0; i < n; ++i) {
        out << rnd() % 100 + 1 << ' ';
    }
    out << '\n';
    for (int i = 0; i < n; ++i) {
        int l = rnd() % 10;
        int r = rnd() % 10 + l;
        out << l << ' ' << r << '\n';
    }
}

signed main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout.tie(0);
#endif
//    while (true) {
//        ostringstream test;
//        GenTest2(test);
//        istringstream in(test.str());
//        ostringstream out;
//        Solve4(in, out);
//        if (out.str() != "-1\n") {
//            cerr << test.str() << '\n';
//            cout << out.str();
//            break;
//        }
//    }
//    StressTest(Solve4, Solve3, GenTest);
    Solve4(cin, cout);
//    SolveStupid(cin, cout);
//    StressTest(SolveStupid, Solve4, GenTest2);
cerr << clock() * 1.0 / CLOCKS_PER_SEC << '\n';
    return 0;
}
