#include <bits/stdc++.h>

using namespace std;

#define int long long

class SegmentTreeMin {
	vector<int> a;
	int n;

public:
	SegmentTreeMin() {}
	SegmentTreeMin(const vector<int>& data) {
		n = data.size();
		a.resize(2 * n);
		copy(data.begin(), data.end(), a.begin() + n);
		for(int i = n - 1; i >= 1; i--) {
			a[i] = min(a[i * 2], a[i * 2 + 1]);
		}
	}

	int get(int l, int r) {
		l += n;
		r += n;
		int ans = n;
		while(l < r) {
			ans = min(ans, a[l]);
			ans = min(ans, a[r - 1]);
			l = (l + 1) / 2;
			r /= 2;
		}
		return ans;
	}
};

struct Node {
	int sum;
	int left, right;
};
vector<Node> nodes;

class SegmentTree {
	vector<int> a;
	int n;

public:
	SegmentTree(int n): n(n) {
		a.resize(4 * n);
	}

	void set(int v, int vl, int vr, int i, int x) {
		if(vr - vl == 1) {
			a[v] = x;
		} else {
			int vm = (vl + vr) / 2;
			if(i < vm) {
				set(v * 2, vl, vm, i, x);
			} else {
				set(v * 2 + 1, vm, vr, i, x);
			}
			a[v] = a[v * 2] + a[v * 2 + 1];
		}
	}

	int get(int v, int vl, int vr, int l, int r) {
		if(vl == l && vr == r) {
			return a[v];
		} else {
			int vm = (vl + vr) / 2;
			if(r <= vm) {
				return get(v * 2, vl, vm, l, r);
			} else if(l >= vm) {
				return get(v * 2 + 1, vm, vr, l, r);
			} else {
				return get(v * 2, vl, vm, l, vm) + get(v * 2 + 1, vm, vr, vm, r);
			}
		}
	}

	int prefix_ge(int v, int vl, int vr, int x) {
		if(x == 0) {
			return vl;
		}
		if(vr - vl == 1) {
			assert(x == 1 && a[v] == 1);
			return vr;
		}
		int vm = (vl + vr) / 2;
		int sum_l = a[v * 2];
		if(x < sum_l) {
			return prefix_ge(v * 2, vl, vm, x);
		} else {
			return prefix_ge(v * 2 + 1, vm, vr, x - sum_l);
		}
	}

	void set(int i, int x) {
		set(1, 0, n, i, x);
	}

	int get(int l, int r) {
		return get(1, 0, n, l, r);
	}

	int prefix_ge(int x) {
		return prefix_ge(1, 0, n, x);
	}
};

int n;
vector<int> a;
SegmentTreeMin a_st;

signed main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);

	cin >> n;

	a.resize(n);
	for(int& x: a) {
		cin >> x;
		x--;
	}

	a_st = SegmentTreeMin(a);

	vector<int> b(n);
	for(int i = 0; i < n; i++) {
		b[a[i]] = i;
	}

	int q;
	cin >> q;

	vector<tuple<int, int, int, int>> queries(q);

	vector<vector<pair<int, int>>> need_queries(n + 1);

	for(int i = 0; i < q; i++) {
		int r1, c1, r2, c2;
		cin >> r1 >> c1 >> r2 >> c2;
		queries[i] = {r1, c1, r2, c2};

		need_queries[r1].emplace_back(r1 - c1, i * 2);
		need_queries[r2].emplace_back(r2 - c2, i * 2 + 1);
	}

	vector<int> answers(q * 2);
	SegmentTree st(n);
	for(int i = 0; i < n; i++) {
		st.set(b[i], 1);
		for(auto [x, k]: need_queries[i + 1]) {
			answers[k] = st.prefix_ge(x);
		}
	}

	for(auto& item: need_queries) {
		item.clear();
	}

	vector<int> r3(q);

	for(int i = 0; i < q; i++) {
		auto [r1, c1, r2, c2] = queries[i];

		int i_u = answers[i * 2];
		int i_v = answers[i * 2 + 1];
		r3[i] = min({a_st.get(min(i_u, i_v), max(i_u, i_v)), r1, r2});

		need_queries[r3[i]].emplace_back(i_u, i);
	}

	answers.clear();
	answers.resize(q);
	SegmentTree st1(n);
	for(int i = 0; i < n; i++) {
		st1.set(b[i], 1);
		for(auto [x, k]: need_queries[i + 1]) {
			answers[k] = st1.get(0, x);
		}
	}

	for(int i = 0; i < q; i++) {
		auto [r1, c1, r2, c2] = queries[i];
		int c3 = r3[i] - answers[i];
		cout << r3[i] << " " << c3 << "\n";
	}

	return 0;
}