#include <vector>
#include <ios>
#include <iostream>
#include <set>
#include <map>
#include <random>
#include <iomanip>
#include <algorithm>
#include <fstream>
#include <math.h>
#include <stack>
#include <queue>

using namespace std;

typedef long long ll;
typedef long double ld;
typedef vector<int> vi;
typedef vector<vi> vvi;

#define all(x) ((x).begin()), ((x).end())
#define rall(x) ((x).rbegin()), ((x).rend())
#define nl '\n'

struct treeB {
	int size;
	vi costs;
	vvi children;
	vi parent;
	vi left, right;

	treeB(int n) {
		size = n;
		costs.resize(n);
		children.resize(n);
		parent.resize(n, -1);
		left.resize(n);
		right.resize(n);
	}

	void addEdge(int c, int p) {
		children[p].push_back(c);
		parent[c] = p;
	}

	bool correctingDFS(int v) {
		ll minSum = 0;
		ll maxSum = 0;
		bool res = true;
		for (auto c : children[v])
		{
			res &= correctingDFS(c);
			minSum += left[c];
			maxSum += right[c];
		}
		if (minSum < 1e9) {
			left[v] = max(left[v], (int)minSum);
		}
		if (right[v] <= left[v]) res = false;
		return res;
	}

	pair<ll, set<pair<int, int>>> dfs(int v) {
		ll minCost = 0;
		ll cnt = 0;
		set<pair<int, int>> options;
		for (auto c : children[v])
		{
			auto res = dfs(c);
			minCost += res.first;
			cnt += left[c];
			options.insert(all(res.second));
		}
		options.insert({ costs[v], right[v] - left[v] });
		int delta = left[v] - cnt;
		while (delta > 0) {
			auto opt = *options.begin();
			options.erase(options.begin());
			if (opt.second > delta) {
				options.insert({ opt.first, opt.second - delta });
				minCost += (ll)delta * (ll)opt.first;
				delta = 0;
			}
			else {
				delta -= opt.second;
				minCost += (ll)opt.first * (ll)opt.second;
			}
		}
		set<pair<int, int>> res;
		if (v != 0) {
			int offset = right[v] - left[v];
			for (auto ptr = options.begin(); ptr != options.end(); ptr++)
			{
				if (offset == 0) break;
				if (ptr->second > offset) {
					res.insert({ ptr->first, offset });
					offset = 0;
				}
				else {
					res.insert(*ptr);
					offset -= ptr->second;
				}
			}
		}

		return { minCost, res };
	}
};

int main() {
	ios::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);

	cout << fixed << setprecision(16);

	int t;
	cin >> t;
	while (t--) {
		int n;
		cin >> n;

		treeB T(n);

		for (int i = 1; i < n; i++)
		{
			int p;
			cin >> p;
			p--;
			T.addEdge(i, p);
		}

		for (int i = 0; i < n; i++)
		{
			cin >> T.costs[i];
		}
		for (int i = 0; i < n; i++)
		{
			cin >> T.left[i];
			cin >> T.right[i];
			T.right[i]++;
		}

		bool res = T.correctingDFS(0);
		if (!res) {
			cout << -1 << nl;
			continue;
		}

		auto ans = T.dfs(0);
		cout << ans.first << nl;

	}
	cout << endl;

	return 0;
}