#include <vector>
#include <ios>
#include <iostream>
#include <set>
#include <map>
#include <random>
#include <iomanip>
#include <algorithm>
#include <fstream>
#include <math.h>
#include <stack>
#include <queue>

using namespace std;

typedef long long ll;
typedef long double ld;
typedef vector<int> vi;
typedef vector<vi> vvi;

#define all(x) ((x).begin()), ((x).end())
#define rall(x) ((x).rbegin()), ((x).rend())
#define nl '\n'

struct treeC {
	vector<set<pair<short, short>>> children;
	vector<vector<int>> bin_up;
	vi tin, tout;
	int size;

	treeC(int n) {
		size = n;
		children.resize(n * n);
		bin_up.resize(n * n);
		tin.resize(n * n);
		tout.resize(n * n);
	}

	void dfs(int id, int parent, int& t) {
		tin[id] = t;
		bin_up[id].push_back(parent);
		for (int i = 1; i < 16; i++)
		{
			bin_up[id].push_back(bin_up[bin_up[id][i - 1]][i - 1]);
		}
		for (auto c : children[id])
		{
			dfs(c.first * size + c.second, id, t);
		}
		t++;
		tout[id] = t;
	}

	bool isAncestor(int v, int p) {
		return (tin[p] <= tin[v] && tout[p] >= tout[v]);
	}

	int lca(int a, int b) {
		if (isAncestor(b, a)) return a;
		int current = a;
		for (int i = 15; i >= 0; i--)
		{
			if (!isAncestor(b, bin_up[current][i])) {
				current = bin_up[current][i];
			}
		}
		return bin_up[current][0];
	}
};

int main() {
	ios::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);

	cout << fixed << setprecision(16);

	int n;
	cin >> n;
	n++;
	
	treeC T(n);

	vi commands(n - 1, 0);

	pair<int, int> cur = { 0, 0 };
	for (int i = 0; i < n - 1; i++)
	{
		pair<int, int> next = cur;

		next.first++;
		if (commands[i] == 0) {
			next.second++;	
		}
		T.children[cur.first * n + cur.second].insert(next);
		cur = next;
	}
	for (int j = 0; j < n - 1; j++)
	{
		int id;
		cin >> id;
		id--;
		commands[id] = 1;
		cur = { 0, 0 };
		for (int i = 0; i < n - 1; i++)
		{
			pair<int, int> next = cur;

			next.first++;
			if (commands[i] == 0) {
				next.second++;
			}
			T.children[cur.first * n + cur.second].insert(next);
			cur = next;
		}
	}
	int t = 0;
	T.dfs(0, 0, t);
	int q;
	cin >> q;

	while (q--) {
		int r1, c1, r2, c2;
		cin >> r1 >> c1 >> r2 >> c2;

		int id1 = r1 * n + c1;
		int id2 = r2 * n + c2;
		int res = T.lca(id1, id2);
		cout << res / n << ' ' << res % n << nl;
	}
	cout.flush();

	return 0;
}