#include <iostream>
#include <algorithm>
#include <set>

struct Node {
    Node* left, *right;
    int32_t sum;
};

int32_t get_sum(Node* node, int32_t left, int32_t right, int32_t query_left, int32_t query_right) {
    if(left >= query_right || right <= query_left)
        return 0;
    if(left >= query_left && right <= query_right)
        return node->sum;
    int32_t m = (left + right) / 2;
    return get_sum(node->left, left, m, query_left, query_right) + get_sum(node->right, m, right, query_left, query_right);
}

Node* update(Node* node, int32_t left, int32_t right, int32_t ind) {
    Node* res = new Node;
    res->sum = node->sum + 1;
    if (right - left == 1) {
        return res;
    }
    int32_t m = (left + right) / 2;
    if(ind >= m) {
        res->left = node->left;
        res->right = update(node->right, m, right, ind);
    } else {
        res->right = node->right;
        res->left = update(node->left, left, m, ind);
    }
    return res;
}

Node* build(int32_t left, int32_t right) {
    Node* res = new Node;
    res->sum = 0;
    if(right - left == 1)
        return res;
    int32_t m = (left + right) / 2;
    res->left = build(left, m);
    res->right = build(m, right);
    return res;
}

int32_t n;
int32_t* arr;
Node** roots;
int32_t position_on_level(int32_t prefix, int32_t level) {
    /*int32_t ans = level;
    for(int32_t i = 0; i < prefix; i++)
        if(arr[i] < level)
            ans--;
    return ans;*/
    return level - get_sum(roots[prefix], 0, n, 0, level);
}

int main() {
    std::ios::sync_with_stdio(false);
    std::cin.tie(0);

    std::cin >> n;

    arr = new int32_t[n];
    for(int32_t i = 0; i < n; i++) {
        std::cin >> arr[i];
        arr[i]--;
    }

    roots = new Node*[n + 1];
    roots[0] = build(0, n);
    for(int32_t i = 1; i <= n; i++) {
        roots[i] = update(roots[i - 1], 0, n, arr[i - 1]);
    }

    int32_t num_q;
    std::cin >> num_q;

    for(int32_t q = 0; q < num_q; q++) {
        int32_t r1, c1, r2, c2;
        std::cin >> r1 >> c1 >> r2 >> c2;

        int32_t left = -1, right = n;
        while(right - left > 1) {
            int32_t m = (left + right) / 2;

            if(position_on_level(m, r1) > c1)
                left = m;
            else
                right = m;
        }
        int32_t v1 = right;

        left = -1, right = n;
        while(right - left > 1) {
            int32_t m = (left + right) / 2;

            if(position_on_level(m, r2) > c2)
                left = m;
            else
                right = m;
        }
        int32_t v2 = right;
        int32_t old_v1 = v1, old_v2 = v2;
        if(v1 > v2)
            std::swap(v1, v2);
        if(v1 == v2) {
            std::cout << (r1 < r2 ? r1 : r2) << " " << (r1 < r2 ? c1 : c2) << "\n";
            continue;
        }
        if(old_v1 < old_v2 && r1 < r2 && position_on_level(old_v2, r1) == c1) {
            std::cout << r1 << " " << c1 << "\n";
            continue;
        }
        if(old_v2 < old_v1 && r2 < r1 && position_on_level(old_v1, r2) == c2) {
            std::cout << r2 << " " << c2 << "\n";
            continue;
        }

        int32_t high_update = n + 100;
        for(int32_t i = v1 + 1; i <= v2; i++)
            high_update = std::min(high_update, arr[i - 1]);
        int32_t ans_level = high_update;
        std::cout << ans_level << " " << position_on_level(v1, ans_level) << "\n";
    }
    return 0;
}
