#ifdef LOCAL
#define _GLIBCXX_DEBUG
#else

#endif

#include <algorithm>
#include <chrono>
#include <deque>
#include <iomanip>
#include <iostream>
#include <map>
#include <numeric>
#include <queue>
#include <random>
#include <set>
#include <unordered_map>
#include <unordered_set>
#include <vector>

using namespace std;
using ll = long long;

struct DP {
    int x, y, i;
    DP(int x, int y, int i) : x(x), y(y), i(i) {}
};

bool operator<(const DP &a, const DP &b) {
    return tie(a.x, a.y, a.i) < tie(b.x, b.y, b.i);
}

struct Graph {
    int n;
    vector<vector<int>> g;
    vector<int> l, r, c;
    vector<ll> mncnt, cnt, mnsm, sm, ans;
    vector<set<DP>> dp;

    Graph(int n)
        : n(n),
          g(n),
          l(n),
          r(n),
          c(n),
          mncnt(n, 0),
          cnt(n, 0),
          mnsm(n, 0),
          sm(n, 0),
          ans(n, 0),
          dp(n) {}

    void add_edge(int v, int u) { g[v].push_back(u); }

    void dfs(int v) {
        int mx = -1;
        for (int u : g[v]) {
            dfs(u);
            mncnt[v] += mncnt[u];
            mnsm[v] += mnsm[u];
            while (!dp[u].empty() && dp[u].rbegin()->x > c[v]) {
                auto [x, y, i] = *dp[u].rbegin();
                dp[u].erase({x, y, i});
                cnt[u] -= y;
                sm[u] -= 1ll * x * y;
            }
            if (mx == -1 || (int)dp[u].size() > (int)dp[mx].size()) mx = u;
        }
        if (mx != -1) {
            dp[v] = move(dp[mx]);
            sm[v] = sm[mx];
            cnt[v] = cnt[mx];
        }
        for (int u : g[v]) {
            if (u == mx) continue;
            for (auto [x, y, i] : dp[u]) {
                dp[v].emplace(x, y, i);
                sm[v] += 1ll * x * y;
                cnt[v] += y;
            }
            dp[u].clear();
        }
        ll rem = l[v] - mncnt[v];
        while (rem > 0 && !dp[v].empty()) {
            auto [x, y, i] = *dp[v].begin();
            dp[v].erase(dp[v].begin());
            if (rem < y) {
                y -= int(rem);
                ans[i] += int(rem);
                mncnt[v] += int(rem);
                mnsm[v] += 1ll * rem * x;
                sm[v] -= 1ll * rem * x;
                cnt[v] -= rem;
                rem = 0;
            } else {
                mncnt[v] += y;
                mnsm[v] += 1ll * y * x;
                sm[v] -= 1ll * y * x;
                cnt[v] -= y;
                rem -= y;
                ans[i] += y;
                y = 0;
            }
            if (y) {
                dp[v].emplace(x, y, i);
            }
        }
        if (rem > 0) {
            mncnt[v] += rem;
            mnsm[v] += 1ll * rem * c[v];
            ans[v] += rem;
        }
        rem = r[v] - cnt[v] - mncnt[v];
        if (rem > 0) {
            dp[v].emplace(c[v], rem, v);
            sm[v] += 1ll * c[v] * rem;
            cnt[v] += rem;
        } else if (rem < 0) {
            rem = -rem;
            while (!dp[v].empty() && rem > 0) {
                auto [x, y, i] = *dp[v].rbegin();
                dp[v].erase({x, y, i});
                if (rem < y) {
                    y -= rem;
                    cnt[v] -= rem;
                    sm[v] -= 1ll * rem * x;
                    rem = 0;
                } else {
                    rem -= y;
                    cnt[v] -= y;
                    sm[v] -= 1ll * y * x;
                    y = 0;
                }
                if (y) {
                    dp[v].emplace(x, y, i);
                }
            }
        }
    }
};

void solve() {
    int n;
    cin >> n;
    Graph g(n);
    for (int j = 1; j < n; j++) {
        int p;
        cin >> p;
        p--;
        g.add_edge(p, j);
    }
    for (int i = 0; i < n; i++) {
        cin >> g.c[i];
    }
    for (int i = 0; i < n; i++) {
        cin >> g.l[i] >> g.r[i];
    }
    g.dfs(0);
    for (int i = 0; i < n; i++) {
        if (g.mncnt[i] > g.r[i]) {
            cout << "-1\n";
            return;
        }
    }
    cout << g.mnsm[0] << '\n';
    for (int i = 0; i < n; i++) {
        cout << g.ans[i] << ' ';
    }
    cout << '\n';
}

int main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cin.exceptions(cin.failbit);
#endif

    int t = 1;
    cin >> t;
    while (t--) solve();
}
