#ifdef LOCAL
#define _GLIBCXX_DEBUG
#else

#endif

#include <algorithm>
#include <chrono>
#include <deque>
#include <iomanip>
#include <iostream>
#include <map>
#include <numeric>
#include <queue>
#include <random>
#include <set>
#include <unordered_map>
#include <unordered_set>
#include <vector>

using namespace std;
using ll = long long;

struct BIT {
    int n;
    vector<int> t;

    BIT(int n) : n(n), t(n + 1, 0) {}

    static int f(int x) { return x & -x; }

    int get(int r) {
        int res = 0;
        for (; r > 0; r -= f(r)) {
            res += t[r];
        }
        return res;
    }
    void update(int i) {
        for (i++; i <= n; i += f(i)) {
            t[i]++;
        }
    }

    int find(int x) {
        int res = 0, sm = 0;
        for (int k = 18; k >= 0; k--) {
            int i = res + (1 << k);
            if (i > n) continue;
            if (sm + t[i] >= x) continue;
            sm += t[i], res = i;
        }
        return res;
    }
};

const int K = 21;

struct Graph {
    int n, t = 0;
    vector<vector<int>> g, up;
    vector<int> tin, tout;

    Graph(int n) : n(n), g(n), up(n, vector<int>(K)), tin(n), tout(n) {
        for (int i = 0; i < n; i++) {
            up[i][0] = i;
        }
    };

    void add_edge(int v, int u) {
        g[v].push_back(u);
        up[u][0] = v;
    }

    void dfs(int v) {
        tin[v] = t++;
        for (int i = 1; i < K; i++) {
            up[v][i] = up[up[v][i - 1]][i - 1];
        }
        for (int u : g[v]) {
            dfs(u);
        }
        tout[v] = t;
    }

    bool is_parent_of(int v, int u) {
        return tin[v] <= tin[u] && tout[u] <= tout[v];
    }

    int lca(int v, int u) {
        if (is_parent_of(v, u)) return v;
        for (int i = K - 1; i >= 0; i--) {
            if (!is_parent_of(up[v][i], u)) v = up[v][i];
        }
        return up[v][0];
    }
};

void solve() {
    int n;
    cin >> n;
    vector<int> a(n);
    BIT hbit(n);
    for (int i = 0; i < n; i++) {
        cin >> a[i];
        a[i]--;
    }
    int q;
    cin >> q;
    int m = n + q * 2;
    vector<vector<int>> qh(n + 1);
    vector<int> b(n), R(m), C(m);
    for (int i = 0; i < n; i++) {
        R[i] = a[i];
        C[i] = hbit.get(a[i]);
        hbit.update(a[i]);
        b[a[i]] = i;
    }
    for (int i = 0; i < q * 2; i++) {
        cin >> R[i + n] >> C[i + n];
        C[i + n] = R[i + n] - C[i + n];
        qh[R[i + n]].push_back(i);
    }
    vector<int> cur(n + 1, -1), p(m, -1);
    BIT vbit(n + 1);
    vbit.update(0);
    for (int i = 0; i < n; i++) {
        int j = b[i];
        int c = C[j];
        int x = vbit.find(c + 1);
        p[j] = cur[x];
        cur[x] = j;
        for (int qq : qh[i]) {
            int x = vbit.find(C[qq + n] + 1);
            p[qq + n] = cur[x];
            cur[x] = qq + n;
        }
        cur[j + 1] = j;
        vbit.update(j + 1);
    }
    for (int qq : qh[n]) {
        p[qq + n] = cur[C[qq + n]];
    }
    Graph g(m);
    int root = -1;
    for (int i = 0; i < m; i++) {
        if (p[i] != -1) {
            g.add_edge(p[i], i);
        } else {
            root = i;
        }
    }
    g.dfs(root);
    for (int i = 0; i < q; i++) {
        int z = g.lca(n + i * 2, n + i * 2 + 1);
        cout << R[z] << ' ' << R[z] - C[z] << '\n';
    }

    return;
}

int main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cin.exceptions(cin.failbit);
#endif

    int t = 1;
    // cin >> t;
    while (t--) solve();
}
