#include <algorithm>
#include <iostream>
#include <numeric>
#include <vector>
#include <set>

#define all(x) x.begin(), x.end()

using namespace std;
using ll = long long;

const int MAXN = 100'000;
const int INF = (int)1e9;

struct MegaStruct
{
	ll mn = 0, mx = 0;
	bool correct = true;

	struct vert
	{
		int mn = 0, mx = INF, cost = 0, ind = 0;

		vert(int mn, int mx, int cost, int ind) : mn(mn), mx(mx), cost(cost), ind(ind) {}

		bool operator<(const vert& v) const
		{
			return make_pair(cost, ind) < make_pair(v.cost, v.ind);
		}
	};

	struct cmp
	{
		bool operator()(const vert* a, const vert* b) const
		{
			return *a < *b;
		}
	};

	set<vert*, cmp> a;

	void merge(const MegaStruct& st)
	{
		for (auto& v : st.a)
		{
			a.insert(v);
		}
		mn += st.mn;
		mx += st.mx;
		correct &= st.correct;
	}

	void update(int l, int r, int cost, int ind)
	{
		if (a.empty())
		{
			mn = l;
			mx = r;
			a.insert(new vert(l, r, cost, ind));
		}
		else
		{
			mx += r;
			a.insert(new vert(0, r, cost, ind));

			auto it = a.begin();
			while (it != a.end() && mn < l)
			{
				ll dif = min((ll)(*it)->mx - (*it)->mn, l - mn);
				(*it)->mn += dif;
				mn += dif;
				++it;
			}		
			
			it = a.end();
			while (it != a.begin() && mx > r)
			{
				--it;
				ll dif = min((ll)(*it)->mx - (*it)->mn, mx - r);
				(*it)->mx -= dif;
				mx -= dif;
			}

			if (mn > r || mn > mx || mx > r || mx < l)
				correct = false;
		}
	}

	ll getMinCost()
	{
		ll res = 0;
		for (auto& v : a)
			res += (ll)v->cost * v->mn;
		return res;
	}

	void outputValues()
	{
		vector<int> val(a.size());
		for (auto& v : a)
			val[v->ind] = v->mn;
		for (auto x : val)
			cout << x << " ";
		cout << "\n";
	}
};

vector<int> g[MAXN];
int c[MAXN];
int lft[MAXN], rt[MAXN];
int sz[MAXN];
MegaStruct structs[MAXN];

void dfs(int u)
{
	sz[u] = 1;
	for (auto v : g[u])
	{
		dfs(v);
		sz[u] += sz[v];
	}

	if (!g[u].empty())
	{
		vector<int> order(g[u].size());
		iota(all(order), 0);
		//sort(all(order), [&](int a, int b) {return sz[g[u][a]] < sz[g[u][b]]; });
		//int f = order.back();
		//order.pop_back();

		//swap(structs[g[u][f]], structs[u]);
		for (auto i : order)
		{
			structs[u].merge(structs[g[u][i]]);
		}
	}

	structs[u].update(lft[u], rt[u], c[u], u);
}

void clear(int n)
{
	for (int i = 0; i < n; ++i)
	{
		g[i].clear();
		structs[i] = MegaStruct();
		sz[i] = 0;
		lft[i] = 0;
		rt[i] = INF;
	}
}

/*
2
3
1 1
3 1 2
5 7
1 2
2 4
2
1
5 5
0 1
2 2

*/

void solve()
{
	int n = 0;
	cin >> n;

	clear(n);

	int p = 0;
	for (int i = 1; i < n; ++i)
	{
		cin >> p;
		g[p - 1].push_back(i);
	}

	for (int i = 0; i < n; ++i)
		cin >> c[i];

	for (int i = 0; i < n; ++i)
		cin >> lft[i] >> rt[i];

	dfs(0);

	if (structs[0].correct)
	{
		cout << structs[0].getMinCost() << "\n";
		structs[0].outputValues();
	}
	else
		cout << -1 << "\n";
}

int main()
{
	ios::sync_with_stdio(false);
	cin.tie(0), cout.tie(0);

	int t = 0;
	cin >> t;

	while (t--)
	{
		solve();
	}

	

	return 0;
}
