#include <bits/stdc++.h>

using namespace std;
using ll = long long;
using pll = pair<ll, ll>;
using pii = pair<int, int>;

const int N = 1 << 19;

struct seg_tree_sum {
    int t[2 * N + 1];

    void update(int i) {
        t[i] = t[2 * i] + t[2 * i + 1];
    }

    void build(int n) {
        for (int i = 0; i < n; ++i) {
            t[i + N] = 1;
        }
        for (int i = N - 1; i >= 1; --i) {
            update(i);
        }
    }

    int _sum(int i, int l, int r, int L, int R) {
        if (L <= l && r <= R) return t[i];
        if (l >= R || r <= L) return 0;
        int m = (l + r) / 2;
        return _sum(2 * i, l, m, L, R) + _sum(2 * i + 1, m, r, L, R);
    }

    int sum(int l, int r) {
        return _sum(1, 0, N, l, r + 1);
    }

    void set(int i) {
        i += N;
        t[i] = 0;
        while (i > 1) {
            i /= 2;
            update(i);
        }
    }
};

struct seg_tree_min {
    int t[2 * N + 1];

    void update(int i) {
        t[i] = min(t[2 * i], t[2 * i + 1]);
    }

    void build(int n, vector<int> &a) {
        for (int i = 0; i < n; ++i) {
            t[i + N] = a[i];
        }
        for (int i = N - 1; i >= 1; --i) {
            update(i);
        }
    }

    int _min(int i, int l, int r, int L, int R) {
        if (L <= l && r <= R) return t[i];
        if (l >= R || r <= L) return 1e9;
        int m = (l + r) / 2;
        return min(_min(2 * i, l, m, L, R), _min(2 * i + 1, m, r, L, R));
    }

    int get_min(int l, int r) {
        return _min(1, 0, N, l, r + 1);
    }
};

vector<int> arr;
seg_tree_min t_min;
seg_tree_sum t_sum;
int n;

struct node {
    ll a, b;
    node() {}
    node(ll a, ll b) {
        this->a = a;
        this->b = b;
    }
    node(ll x) {
        a = x % (n + 1);
        b = x / (n + 1);
    }
    ll to_int() const {
        return b * (n + 1) + a;
    }
    bool operator<(const node &other) const {
        if ((a - b) != (other.a - other.b))
            return (a - b) < (other.a - other.b);
        return a > other.a;
    }
    bool operator==(const node &other) const {
        return (a == other.a) && (b == other.b);
    }
};

node node_from_path(int ones, int zeroes) {
    return {ones + zeroes, ones};
}

vector<pair<node, node>> queries;
vector<node> sorted_nodes;
unordered_map<ll, int> activation_time;
unordered_map<ll, bool> used;
vector<int> time_to_queries[N];
node ans[N];

bool node_in_path(node& x, seg_tree_sum &t) {
    if (x.a == 0) return true;
    return t.sum(0, x.a - 1) == x.b;
}

void solve() {
    cin >> n;
    for (int i = 0; i < n; ++i) {int x; cin >> x; --x; arr.push_back(x);}

    t_sum.build(n);
    t_min.build(n, arr);

    int q; cin >> q;
    for (int i = 0; i < q; ++i) {
        int a, b, c, d; cin >> a >> b >> c >> d;
        queries.push_back({{a, b}, {c, d}});
        sorted_nodes.emplace_back(a, b);
        sorted_nodes.emplace_back(c, d);
    }
    sort(sorted_nodes.begin(), sorted_nodes.end());

//    for (node nd : sorted_nodes) {
//        cout << nd.a << " " << nd.b << endl;
//    }

    if (n < 1e4) {
        for (int i = 0; i < n + 1; ++i) {
            for (node nd : sorted_nodes) {
                if (!used[nd.to_int()] && node_in_path(nd, t_sum)) {
                    activation_time[nd.to_int()] = i;
                    used[nd.to_int()] = true;
                }
            }
            if (i < n) t_sum.set(arr[i]);
        }
    }
    else {
        int p = 0;
        for (int i = 0; i < n + 1; ++i) {
            while (p < sorted_nodes.size() && node_in_path(sorted_nodes[p], t_sum)) {
                activation_time[sorted_nodes[p].to_int()] = i;
                ++p;
            }
            if (i < n) t_sum.set(arr[i]);
        }
    }


//    for (node nd : sorted_nodes) {
//        cout << nd.a << " " << nd.b << " " << activation_time[nd.to_int()] << endl;
//    }

    for (int i = 0; i < q; ++i) {
        node n1 = queries[i].first, n2 = queries[i].second;
        int time = min(activation_time[n1.to_int()], activation_time[n2.to_int()]);
        time_to_queries[time].push_back(i);
    }

    t_sum.build(n);
    for (int i = 0; i < n + 1; ++i) {
        for (int query : time_to_queries[i]) {
            node n1 = queries[query].first, n2 = queries[query].second;
            int t1 = activation_time[n1.to_int()], t2 = activation_time[n2.to_int()];
            if (t2 < t1) swap(t1, t2);
            if (t1 == t2) {
                ans[query] = n1.a > n2.a ? n2 : n1;
                continue;
            }
            int min_set = t_min.get_min(t1, t2 - 1) - 1;
            if (min_set == -1) ans[query] = {0, 0};
            else {
                int ones = t_sum.sum(0, min_set);
                int zeroes = min_set + 1 - ones;
                ans[query] = node_from_path(ones, zeroes);
                if (ans[query].a > n1.a) ans[query] = n1;
                if (ans[query].a > n2.a) ans[query] = n2;
            }
        }
        if (i < n) t_sum.set(arr[i]);
    }

    for (int i = 0; i < q; ++i) {
        cout << ans[i].a << " " << ans[i].b << '\n';
    }
}

int main() {

    int t; t = 1;
    for (int i = 0; i < t; ++i) solve();

    return 0;
}
