//#define _CRT_SECURE_NO_WARNINGS
//#pragma GCC target ("avx, avx2, sse, sse2, sse3, sse4, fma, popcnt")
//#pragma GCC optimize ("-Ofast, unroll-loops, fast-math")
#include <iostream>
#include <vector>
#include <algorithm>
#include <set>
#include <random>

#define all(a) a.begin(), a.end()
#define rall(a) a.rbegin(), a.rend()

using namespace std;

mt19937 gen(0);

struct node {
	int c, lft, r, ind, sz;
	long long sm;
	node* left, * right;
	node(int c, int lft, int ind) : c(c), lft(lft), ind(ind) {
		sm = lft;
		sz = 1;
		r = gen();
		left = right = nullptr;
	}
};

long long get_sm(node* u) {
	return u == nullptr ? 0 : u->sm;
}
int get_sz(node* u) {
	return u == nullptr ? 0 : u->sz;
}

void update(node* u) {
	u->sm = u->lft + get_sm(u->left) + get_sm(u->right);
	u->sz = 1 + get_sz(u->left) + get_sz(u->right);
}

node* merge(node* u, node* v) {
	if (u == nullptr)
		return v;
	if (v == nullptr)
		return u;
	if (u->r < v->r) {
		u->right = merge(u->right, v);
		update(u);
		return u;
	}
	else {
		v->left = merge(u, v->left);
		update(v);
		return v;
	}
}

pair<node*, node*> split(node* u, int c, int ind) {
	if (u == nullptr)
		return { nullptr, nullptr };
	if (tie(u->c, u->ind) <= tie(c, ind)) {
		auto [l, r] = split(u->right, c, ind);
		u->right = l;
		update(u);
		return { u, r };
	}
	else {
		auto [l, r] = split(u->left, c, ind);
		u->left = r;
		update(u);
		return { l, u };
	}
}

vector<vector<int>> gr;
vector<int> c, l, r, ans;

node* add(node* u, node* nd) {
	if (u == nullptr)
		return nd;
	if (u->r < nd->r && tie(u->c, u->ind) < tie(nd->c, nd->ind)) {
		u->right = add(u->right, nd);
		update(u);
		return u;
	}
	else if (u->r < nd->r) {
		u->left = add(u->left, nd);
		update(u);
		return u;
	}
	else {
		auto [l, r] = split(u, nd->c, nd->ind);
		nd->left = l;
		nd->right = r;
		update(nd);
		return nd;
	}
}

node* buy(node* u, int& nd) {
	if (u == nullptr)
		return nullptr;
	node* l = buy(u->left, nd);
	if (nd > 0) {
		u->left = nullptr;
		if (nd < u->lft) {
			u->lft -= nd;
			ans[u->ind] += nd;
			nd = 0;
			update(u);
			return u;
		}
		else if (nd == u->lft) {
			ans[u->ind] += nd;
			nd = 0;
			return u->right;
		}
		else {
			nd -= u->lft;
			ans[u->ind] += u->lft;
			u->lft = 0;
			node* r = buy(u->right, nd);
			return r;
		}
	}
	else {
		u->left = l;
		update(u);
		return u;
	}
}

node* addall(node* u, node* to) {
	if (u == nullptr)
		return to;
	to = addall(u->left, to);
	to = addall(u->right, to);
	u->left = u->right = nullptr;
	update(u);
	to = add(to, u);
	return to;
}

node* slice(node* u, long long val) {
	if (u == nullptr)
		return nullptr;
	long long sm = get_sm(u->left);
	if (sm > val)
		return slice(u->left, val);
	if (sm == val)
		return u->left;
	if (sm + u->lft >= val) {
		u->lft -= sm + u->lft - val;
		u->right = nullptr;
		update(u);
		return u;
	}
	u->right = slice(u->right, val - sm - u->lft);
	update(u);
	return u;
}

long long dfs_check(int u) {
	long long mn = 0, mx = 0;
	for (int v : gr[u]) {
		long long t = dfs_check(v);
		if (t == -1)
			return -1;
		mn += t;
	}
	if (mn > r[u])
		return -1;
	return max(mn, 1ll * l[u]);
}

pair<node*, long long> dfs(int u) {
	node* s = new node(c[u], r[u], u);
	long long bt = 0;
	for (int v : gr[u]) {
		auto [ns, nbt] = dfs(v);
		if (get_sz(ns) > get_sz(s)) {
			swap(s, ns);
			swap(bt, nbt);
		}
		bt += nbt;
		addall(ns, s);
	}
	if (bt < l[u]) {
		int nd = l[u] - bt;
		s = buy(s, nd);
		bt = l[u];
	}
	s = slice(s, r[u] - bt);
	return { s, bt };
}

int main()
{
	ios::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);
	int t;
	cin >> t;
	while (t--) {
		int n;
		cin >> n;
		gr.assign(n, {});
		c.assign(n, 0);
		l.assign(n, 0);
		r.assign(n, 0);
		for (int i = 1; i < n; ++i) {
			int temp;
			cin >> temp;
			gr[temp - 1].push_back(i);
		}
		for (int i = 0; i < n; ++i)
			cin >> c[i];
		for (int i = 0; i < n; ++i) {
			cin >> l[i] >> r[i];
		}
		if (dfs_check(0) == -1) {
			cout << "-1\n";
			continue;
		}
		ans.assign(n, 0);
		dfs(0);
		long long res = 0;
		for (int i = 0; i < n; ++i)
			res += 1ll * c[i] * ans[i];
		cout << res << '\n';
		for (int i = 0; i < n; ++i)
			cout << ans[i] << ' ';
		cout << '\n';
	}
}
