#define _GLIBCXX_DEBUG
#include <bits/stdc++.h>

using namespace std;
using ll = long long;

void solve();

int main() {
    cin.tie(nullptr);
    cout.tie(nullptr);
    ios_base::sync_with_stdio(false);
    solve();
    return 0;
}

#define lower_index(v, x) lower_bound(v.begin(), v.end(), x) - v.begin()
#define upper_index(v, x) upper_bound(v.begin(), v.end(), x) - v.begin()

struct cell {
    int i, j;

    cell(int i, int j) : i(i), j(j) {}

    cell() = default;

    bool operator<(const cell &other) const {
        return pair{i, j} < pair{other.i, other.j};
    }
};

struct SumTree {
    int n;
    vector<int> s;

    SumTree(int sz) : n(2 << __lg(sz)), s(2 * n) {}

    void set(int i, int x) {
        s[i += n] = x;
        for (i /= 2; i > 0; i /= 2) s[i] = s[2 * i] + s[2 * i + 1];
    }

    int get(int l, int r) {
        int res = 0;
        for (l += n, r += n; l < r; l /= 2, r /= 2) {
            if (l & 1) res += s[l++];
            if (r & 1) res += s[--r];
        }
        return res;
    }

    int atLeast(int sum) {
        int i = 1;
        while (i < n) {
            if (s[2 * i] >= sum) {
                i = 2 * i;
            } else {
                sum -= s[2 * i];
                i = 2 * i + 1;
            }
        }
        return i - n + 1;
    }
};

struct MinTree {
    int n;
    vector<int> s;

    MinTree(int sz) : n(2 << __lg(sz)), s(2 * n) {}

    void set(int i, int x) {
        s[i += n] = x;
        for (i /= 2; i > 0; i /= 2) s[i] = min(s[2 * i], s[2 * i + 1]);
    }

    int get(int l, int r) {
        int res = 1e9;
        for (l += n, r += n; l < r; l /= 2, r /= 2) {
            if (l & 1) res = min(res, s[l++]);
            if (r & 1) res = min(res, s[--r]);
        }
        return res;
    }
};

void solve(vector<int> a, vector<pair<cell, cell>> q) {
    int n = a.size();
    vector<int> pos(n);
    for (int i = 0; i < n; ++i) pos[a[i]] = i;
    vector<tuple<int, int, int>> q1;
    q1.reserve(2 * q.size());
    for (int idx = 0; idx < q.size(); ++idx) {
        auto &[c1, c2] = q[idx];
        auto[i1, j1] = c1;
        auto[i2, j2] = c2;
        q1.emplace_back(i1, j1, 2 * idx);
        q1.emplace_back(i2, j2, 2 * idx + 1);
    }
    vector<int> ans1(q1.size());
    {
        SumTree st(n);
        sort(q1.begin(), q1.end());
        int curI = 0;
        for (auto &[i, j, idx]: q1) {
            while (i > curI) st.set(pos[curI++], 1);
            ans1[idx] = (i == j ? 0 : st.atLeast(i - j));
        }
    }
    MinTree mt(n);
    for (int i = 0; i < n; ++i) mt.set(i, a[i]);
    vector<tuple<int, int, int>> q2;
    q2.reserve(q.size());
    vector<int> ans3(q.size());
    for (int idx = 0; idx < q.size(); ++idx) {
        auto &[c1, c2] = q[idx];
        auto[i1, j1] = c1;
        auto[i2, j2] = c2;
        int idx1 = ans1[2 * idx];
        int idx2 = ans1[2 * idx + 1];
        int ans = min({i1, i2, mt.get(min(idx1, idx2), max(idx1, idx2))});
        ans3[idx] = ans;
        q2.emplace_back(ans, idx1, idx);
    }
    vector<int> ans2(q2.size());
    {
        SumTree st(n);
        sort(q2.begin(), q2.end());
        int curI = 0;
        for (auto &[i, c, idx]: q2) {
            while (i > curI) st.set(pos[curI++], 1);
            ans2[idx] = i - st.get(0, c);
        }
    }
    for (int idx = 0; idx < q.size(); ++idx) {
        auto &[c1, c2] = q[idx];
        int ans = ans3[idx];
        cout << ans << ' ' << ans2[idx] << '\n';
    }
}

void solve() {
    int n, q;
    cin >> n;
    vector<int> a(n);
    for (int &x: a) cin >> x;
    for (int &x: a) --x;
    cin >> q;
    vector<pair<cell, cell>> qu(q);
    for (auto &[c1, c2]: qu) {
        cin >> c1.i >> c1.j >> c2.i >> c2.j;
    }
    solve(a, qu);
}
