#include <bits/stdc++.h>

using namespace std;

typedef long long ll;

#define vi vector<int>
#define vvi vector<vi>
#define vll vector<ll>
#define pll pair<ll, ll>
#define pi pair<int, int>
#define vpi vector<pi>
#define vpll vector<pll>
#define vb vector<bool>

struct Node {
    vi elements;
    int n;

    Node() {}

    explicit Node(int num) {
        elements = {num};
        n = 1;
    }

    Node(Node &a, Node &b) {
        int i1 = 0, i2 = 0;
        while (true) {
            if (i1 == a.n and i2 == b.n) break;
            else if (i1 == a.n) elements.push_back(b.elements[i2++]);
            else if (i2 == b.n) elements.push_back(a.elements[i1++]);
            else if (a.elements[i1] < b.elements[i2]) elements.push_back(a.elements[i1++]);
            else elements.push_back(b.elements[i2++]);
        }
        n = a.n + b.n;
    }

    int cnt_less(int x) {
        if (elements[0] > x) return 0;
        int left = 0, right = n, medium;
        while (right - left > 1) {
            medium = (left + right) / 2;
            if (elements[medium] <= x) left = medium;
            else right = medium;
        }
        return left + 1;
    }
};

struct MergeSort {
    vector<Node> tree;
    int n;

    explicit MergeSort(vi &arr) {
        n = arr.size();
        tree.resize(4 * n);
        build(1, 0, n - 1, arr);
    }

    void build(int i, int tl, int tr, vi &arr) {
        if (tl == tr) tree[i] = Node(arr[tl]);
        else {
            int tm = (tl + tr) / 2;
            build(i * 2, tl, tm, arr);
            build(i * 2 + 1, tm + 1, tr, arr);
            tree[i] = Node(tree[i * 2], tree[i * 2 + 1]);
        }
    }

    int find(int i, int tl, int tr, int val, int cnt) {
        if (tl == tr) return tl;
        int tm = (tl + tr) / 2;
        int res = tree[i * 2].cnt_less(val);
        if (res >= cnt) return find(i * 2, tl, tm, val, cnt);
        return find(i * 2 + 1, tm + 1, tr, val, cnt - res);
    }

    int calc_lower(int i, int tl, int tr, int l, int r, int val) {
        if (tl == l and tr == r) return tree[i].cnt_less(val);
        int tm = (tl + tr) / 2;
        if (r <= tm) return calc_lower(i * 2, tl, tm, l, r, val);
        else if (l > tm) return calc_lower(i * 2 + 1, tm + 1, tr, l, r, val);
        return calc_lower(i * 2, tl, tm, l, tm, val) + calc_lower(i * 2 + 1, tm + 1, tr, tm + 1, r, val);
    }

    int calc_lower(int l, int r, int val) {
        return calc_lower(1, 0, n - 1, l, r, val);
    }

    int find(int val, int cnt) {
        return find(1, 0, n - 1, val, cnt);
    }
};

struct SegmentTree {
    int n;
    vi tree;

    SegmentTree(vi &arr) {
        n = arr.size();
        tree.resize(4 * n, 0);
        build(1, 0, n - 1, arr);
    }

    void build(int i, int tl, int tr, vi &arr) {
        if (tl == tr) tree[i] = arr[tl];
        else {
            int tm = (tl + tr) / 2;
            build(i * 2, tl, tm, arr);
            build(i * 2 + 1, tm + 1, tr, arr);
            tree[i] = min(tree[i * 2], tree[i * 2 + 1]);
        }
    }

    int get_min(int i, int tl, int tr, int l, int r) {
        if (tl == l and tr == r) return tree[i];
        int tm = (tl + tr) / 2;
        if (r <= tm) return get_min(i * 2, tl, tm, l, r);
        else if (l > tm) return get_min(i * 2 + 1, tm + 1, tr, l, r);
        return min(get_min(i * 2, tl, tm, l, tm),
                   get_min(i * 2 + 1, tm + 1, tr, tm + 1, r));
    }

    int get_min(int l, int r) {
        return get_min(1, 0, n - 1, l, r);
    }
};

int get_min_route(int x, int y, vi &a, MergeSort &ms) {
    if (0 == y - x) return 0;
    return ms.find(y - 1, y - x) + 1;
}

pair<int, int> get_inter(int max_h, int r1, int r2, MergeSort &ms, SegmentTree &st) {
    int m_val = st.get_min(min(r1, r2), max(r1, r2) - 1);
    m_val = min(m_val, max_h - 1);
    int cur_x = m_val - ms.calc_lower(0, max(r1, r2) - 1, m_val - 1), cur_y = m_val;
    return {cur_x, cur_y};
}

void solve() {
    int n;
    cin >> n;
    vi a(n);
    for (int &i: a) {
        cin >> i;
        i--;
    }

    MergeSort ms(a);
    SegmentTree st(a);

    int q;
    cin >> q;
    while (q--) {
        int y1, x1, y2, x2;
        cin >> y1 >> x1 >> y2 >> x2;
        int r1 = get_min_route(x1, y1, a, ms);
        int r2 = get_min_route(x2, y2, a, ms);
        pi p;
        if (r1 > 0 or r2 > 0) p = get_inter(min(y1, y2), r1, r2, ms, st);
        else {
            if (y1 < y2) {
                p = {x1, y1};
            } else {
                p = {x2, y2};
            }
        }
        cout << p.second << ' ' << p.first << '\n';
    }
}

signed main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);
    int tests = 1;
   // cin >> tests;
    while (tests--) {
        solve();
    }
}