#include <iostream>
#include <vector>
#include<math.h>
#include<algorithm>
#include <string>
#include <cstring>
#include <tuple>
#include <queue>

using namespace std;

typedef pair<int, int> pii;

const int INF = 1e9;

vector<vector<int>> gr;
int timer = 0;
vector<int> tin, tout, fup, used;

bool ancestor(int v, int u) {
	return tin[v] <= tin[u] && tout[v] >= tout[u];
}

void dfs(int v) {
	used[v] = 1;
	tin[v] = timer++;
	fup[v] = tin[v];
	for (int u : gr[v]) {
		if (!used[u]) {
			dfs(u);
			fup[v] = min(fup[v], fup[u]);
		}
		else {
			fup[v] = min(fup[v], tin[u]);
		}
	}
	tout[v] = timer++;
}

bool cycle(int v, int u) {
	if (tin[u] < tin[v])
		swap(v, u);
	return fup[u] <= tin[v];
}

void solve() { //todo init
	int n, m, v, u;
	cin >> n >> m;
	gr.resize(n);
	tin.resize(n);
	used.assign(n, 0);
	tout.resize(n);
	fup.resize(n);
	vector<pii> edges;
	for (int i = 0; i < m; i++) {
		cin >> v >> u;
		v--, u--;
		gr[v].push_back(u);
		gr[u].push_back(v);
		edges.emplace_back(v, u);
	}
	dfs(0);
	vector<vector<int>> t(n);
	for (int i = 0; i < m; i++) {
		int v, f, u;
		tie(v, f) = edges[i];
		for (auto u : gr[f]) {
			if (u == v) continue;
			if (cycle(v, u)) {
				t[v].push_back(u);
				t[u].push_back(v);
			}
		}
		swap(v, f);
		for (auto u : gr[f]) {
			if (u == v) continue;
			if (cycle(v, u)) {
				t[v].push_back(u);
				t[u].push_back(v);
			}
		}
	}
	vector<int> dist(n, INF);
	dist[0] = 0;
	queue<int> q;
	q.push(0);
	while (!q.empty()) {
		int v = q.front();
		q.pop();
		for (auto u : t[v]) {
			if (dist[u] > dist[v] + 1) {
				dist[u] = dist[v] + 1;
				q.push(u);
			}
		}
	}
	for (int v = 1; v < n; v++) {
		if (dist[v] == INF)
			cout << "-1 ";
		else
			cout << 2 * dist[v] << ' ';
	}
	cout << '\n';
}

void init() {
	timer = 0;
	tin.clear();
	tout.clear();
	fup.clear();
	gr.clear();
	used.clear();
}

int main()
{
	int t;
	cin >> t;
	while (t--) {
		init();
		solve();
	}
}