#include <iostream>
#include <vector>
#include <algorithm>
#include <map>
#include <random>

using namespace std;

mt19937 twister(1488);
uniform_int_distribution<int> distr(1, 10000000);

struct Treap {
    Treap *left, *right;
    int64_t sum;
    int64_t value;
    int size;

    Treap(int64_t val) {
        value = sum = val;
        size = 1;
        left = right = nullptr;
    }
};

using tree_t = Treap*;

int get_size(tree_t tree) {
    if (tree == nullptr) return 0;
    return tree->size;
}

int64_t get_sum(tree_t tree) {
    if (tree == nullptr) return 0;
    return tree->sum;
}

void update(tree_t tree) {
    if (tree == nullptr);
    tree->size = get_size(tree->left) + get_size(tree->right) + 1;
    tree->sum = get_sum(tree->left) + get_sum(tree->right) + tree->value;
}

tree_t copy(tree_t tree) {
    return new Treap(*tree);
}

pair<tree_t, tree_t> split(tree_t tree, int64_t k) {
    if (tree == nullptr) return {nullptr, nullptr};
    tree = copy(tree);
    if (tree->value < k) {
        auto tmp = split(tree->right, k);
        tree->right = tmp.first;
        update(tree);
        return {tree, tmp.second};
    } else {
        auto tmp = split(tree->left, k);
        tree->left = tmp.second;
        update(tree);
        return {tmp.first, tree};
    }
}

tree_t merge(tree_t a, tree_t b) {
    if (a == nullptr) return b;
    if (b == nullptr) return a;
    uniform_int_distribution<int> distr(get_size(a) + get_size(b));
    int rnd = distr(twister);
    if (rnd < get_size(a)) {
        a = copy(a);
        a->right = merge(a->right, b);
        update(a);
        return a;
    } else {
        b = copy(b);
        b->left = merge(a, b->left);
        update(b);
        return b;
    }
}

tree_t insert(tree_t tree, int64_t x) {
    auto tmp = split(tree, x);
    return merge(tmp.first, merge(new Treap(x), tmp.second));
}


int main() {
    int n, q; cin >> n >> q;
    vector<int64_t> as(n);
    for (int i = 0; i < n; i++) {
        cin >> as[i];
    }
    vector<int64_t> as_copy = as;
    sort(as.begin(), as.end());
    // as.resize(unique(as.begin(), as.end()) - as.begin());

    tree_t current_deltas = nullptr;
    map<int64_t, tree_t> deltas_map;
    for (int i = n - 1; i >= 0; i--) {
        deltas_map[as[i]] = current_deltas;
        if (i != 0) {
            current_deltas = insert(current_deltas, as[i] - as[i - 1]);
        }
    }
    vector<pair<int64_t, int>> deltas;
    for (int i = 0; i < n - 1; i++) {
        deltas.emplace_back(as[i + 1] - as[i], i + 1);
    }
    sort(deltas.begin(), deltas.end());
    vector<int64_t> deltas_pref_sums(n);
    deltas_pref_sums[0] = 0;
    vector<int64_t> deltas_suff_sums_partial(n);
    deltas_suff_sums_partial[n - 1] = 0;
    for (int i = n - 2; i >= 0; i--) {
        deltas_suff_sums_partial[i] = deltas_suff_sums_partial[i + 1] + deltas[i].second;
    }
    for (int i = 1; i < n; i++) {
        deltas_pref_sums[i] = deltas_pref_sums[i - 1] + (deltas[i - 1].second * 1ll * deltas[i - 1].first);
    }
    while (q--) {
        int t; cin >> t;
        if (t == 1) {
            int64_t k; cin >> k;
            int idx = upper_bound(deltas.begin(), deltas.end(), make_pair(k, n + 1488)) - deltas.begin();
            cout << n - idx << endl;
        } else if (t == 2) {
            int64_t k; cin >> k;
            int idx = upper_bound(deltas.begin(), deltas.end(), make_pair(k, n + 1488)) - deltas.begin();
            int64_t ans = deltas_pref_sums[idx] + deltas_suff_sums_partial[idx] * 1ll * k;
            cout << ans << endl;
        } else if (t == 3) {
            int64_t k; cin >> k;
            int i; cin >> i; i--;
            int64_t val = as_copy[i];
            tree_t our_deltas = deltas_map[val];
            int64_t ans = 0;
            auto tmp = split(our_deltas, k);
            ans += get_sum(tmp.first);
            ans += get_size(tmp.second) * 1ll * k;
            cout << ans + val << endl;
        }
    }
}
