#include <iostream>
#include <vector>
#include <functional>
#include <deque>
#include <map>

using namespace std;
using graph_t = vector<vector<int>>;

const int INF = 1000000007;

struct DSU {
    vector<int> parents, sizes;

    DSU(int n) {
        parents.resize(n);
        sizes.assign(n, 1);
        for (int i = 0; i < n; i++) {
            parents[i] = i;
        }
    }

    int find(int v) {
        if (parents[v] == v) {
            return v;
        }
        int u = find(parents[v]);
        parents[v] = u;
        return u;
    }

    void merge(int v, int u) {
        v = find(v);
        u = find(u);
        if (v == u) {
            return;
        }
        if (sizes[v] >= sizes[u]) swap(v, u);
        parents[v] = u;
        sizes[u] += sizes[v];
    }
};


void solve() {
    int n, m; cin >> n >> m;
    graph_t graph(n);
    graph_t edges_numbers_inv(n);
    graph_t edges_numbers(n);
    map<pair<int, int>, int> edges;
    int cnt = 0;
    for (int i = 0; i < m; i++) {
        int v, u; cin >> v >> u; v--; u--;
        graph[u].emplace_back(v);
        graph[v].emplace_back(u);
        edges_numbers_inv[v].emplace_back(cnt);
        edges_numbers[u].emplace_back(cnt);
        cnt++;
        // edges[{u, v}] = cnt++;
        edges_numbers_inv[u].emplace_back(cnt);
        edges_numbers[v].emplace_back(cnt);
        cnt++;
        // edges[{v, u}] = cnt++;
    }
    for (int i = 0; i < n; i++) {
        // edges[{i, i}] = cnt++;
        cnt++;
    }
    int max_states = cnt;
    DSU dsu(max_states);
    for (int v = 0; v < n; v++) {
        // int s0 = edges[{v, graph[v][0]}];
        int s0 = edges_numbers[v][0];
        // for (int u : graph[v]) {
        //     dsu.merge(edges[{v, u}], s0);
        // }
        for (int i = 0; i < (int) graph[v].size(); i++) {
            dsu.merge(edges_numbers[v][i], s0);
        }
    }
    int state_cmps_number = cnt;
    vector<vector<int>> compressed_state_graph(state_cmps_number);
    // set<pair<int, int>> edges;
    // Add non-free transfers
    for (int v = 0; v < n; v++) {
        // int a = dsu.find(edges[{v, v}]);
        int a = 2 * m + v;
        for (int k : edges_numbers_inv[v]) {
            int b = dsu.find(k);
            // int b = dsu.find(edges[{u, v}]);
            // if (edges.find({a, b}) != edges.end()) continue;
            // edges.insert({a, b});
            compressed_state_graph[a].emplace_back(b);
            compressed_state_graph[b].emplace_back(a);
        }
    }
    // find shortest paths
    vector<int> dists(state_cmps_number, INF);
    dists[dsu.find(2 * m)] = 0;
    deque<int> q = {dsu.find(2 * m)};
    while (!q.empty()) {
        int v = q.front();
        q.pop_front();
        for (int u : compressed_state_graph[v]) {
            if (dists[u] > dists[v] + 1) {
                dists[u] = dists[v] + 1;
                q.push_back(u);
            }
        }
    }
    for (int i = 1; i < n; i++) {
        int d = dists[dsu.find(2 * m + i)];
        if (d == INF) {
            cout << -1 << ' ';
        } else {
            cout << d << ' ';
        }
    }
    cout << '\n';
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    int t; cin >> t;
    while (t--) {
        solve();
    }
}
