#include <iostream>
#include <vector>
#include <functional>
#include <deque>
#include <set>

using namespace std;
using graph_t = vector<vector<int>>;

const int INF = 1000000007;

void fill_transfers(const graph_t &graph, graph_t &state_graph, int t) {
    // Excluding t from graph
    int n = graph.size();
    vector<int> cmps(n, -1);
    int timer = 0;
    function<void(int)> fill_cmps = [&](int v) {
        if (v == t) return;
        cmps[v] = timer;
        for (int u : graph[v]) {
            if (cmps[u] != timer) {
                fill_cmps(u);
            }
        }
    };
    for (int i = 0; i < n; i++) {
        if (cmps[i] == -1) {
            fill_cmps(i);
            timer++;
        }
    }
    for (int u : graph[t]) {
        for (int v : graph[t]) {
            if (cmps[u] == cmps[v]) {
                state_graph[t * n + u].emplace_back(t * n + v);
                state_graph[t * n + v].emplace_back(t * n + u);
            }
        }
    }

}

void solve() {
    int n, m; cin >> n >> m;
    graph_t graph(n);
    for (int i = 0; i < m; i++) {
        int v, u; cin >> v >> u; v--; u--;
        graph[u].emplace_back(v);
        graph[v].emplace_back(u);
    }
    int max_states = n * n;
    graph_t state_graph(max_states);
    for (int i = 0; i < n; i++) {
        fill_transfers(graph, state_graph, i);
    }
    vector<int> state_cmps(max_states, -1);
    int timer = 0;
    function<void(int)> fill_cmps = [&](int v) {
        state_cmps[v] = timer;
        for (int u : state_graph[v]) {
            if (state_cmps[u] != timer) {
                fill_cmps(u);
            }
        }
    };
    for (int i = 0; i < max_states; i++) {
        if (state_cmps[i] == -1) {
            fill_cmps(i);
            timer++;
        }
    }
    int state_cmps_number = timer;
    vector<vector<int>> compressed_state_graph(state_cmps_number);
    set<pair<int, int>> edges;
    // Add non-free transfers
    for (int v = 0; v < n; v++) {
        for (int u : graph[v]) {
            int a = state_cmps[v * n + v], b = state_cmps[u * n + v];
            if (edges.find({a, b}) != edges.end()) continue;
            edges.insert({a, b});
            compressed_state_graph[a].emplace_back(b);
            compressed_state_graph[b].emplace_back(a);
        }
    }
    // find shortest paths
    vector<int> dists(state_cmps_number, INF);
    dists[state_cmps[0]] = 0;
    deque<int> q = {state_cmps[0]};
    while (!q.empty()) {
        int v = q.front();
        q.pop_front();
        for (int u : compressed_state_graph[v]) {
            if (dists[u] > dists[v] + 1) {
                dists[u] = dists[v] + 1;
                q.push_back(u);
            }
        }
    }
    for (int i = 1; i < n; i++) {
        int d = dists[state_cmps[i * n + i]];
        if (d == INF) {
            cout << -1 << ' ';
        } else {
            cout << d << ' ';
        }
    }
    cout << endl;
}

int main() {
    int t; cin >> t;
    while (t--) {
        solve();
    }
}
