#include <bits/stdc++.h>

typedef long long ll;
typedef long double ld;
using namespace std;

struct Query {
    int index, t, k, i;
    Query(int index, int t, int k): index(index), t(t), k(k) {}
    Query(int index, int t, int k, int i): index(index), t(t), k(k), i(i) {}
};

struct Fenwick {
    int n;
    vector <int> tree;

    Fenwick(int n): n(n), tree(n) {}

    void add(int r, int dt) {
        for (; r < n; r |= (r + 1))
            tree[r] += dt;
    }

    int sum(int r) {
        int result = 0;
        for (; r >= 0; r = (r & (r + 1)) - 1)
            result += tree[r];
        return result;
    }

    int sum(int l, int r) {
        return sum(r) - sum(l - 1);
    }
};

struct DSU {
    vector <int> parents, size;

    DSU(int n): parents(n), size(n, 1) {
        iota(parents.begin(), parents.end(), 0);
    }

    int get(int v) {
        if (v == parents[v]) return v;
        return parents[v] = get(parents[v]);
    }

    void merge(int u, int v) {
        u = get(u), v = get(v);

        if (u == v) return;
        if (size[u] < size[v]) swap(u, v);

        parents[v] = u;
        size[u] += size[v];
    }
};

void solve() {
    int n, q;
    cin >> n >> q;

    vector <ll> exp(n);
    for (ll &x: exp) cin >> x;

    DSU dsu(n);
    set <pair <int, int>> asse;
    for (int i = 0; i < n; i++)
        asse.emplace(exp[i], i);

    auto step = [&] () {
        ll s = 0;
        map <int, vector <int>> sets;

        ll cnt = 0, cnt2 = 0, pr = -1;
        while (!asse.empty()) {
            auto [x, i] = *asse.rbegin();
            asse.erase(prev(asse.end()));

            if (x != pr) {
                cnt += cnt2;
                cnt2 = 1;
            }

            pr = x;

            // cout << x << " -> " << x + cnt << '\n';
            s += cnt * dsu.size[i];
            x += cnt;

            sets[x].push_back(i);
            exp[i] = x;
        }

        for (auto [k, v]: sets) {
            for (int i = 1; i < v.size(); i++) {
                dsu.merge(v[0], v[i]);
            }

            asse.emplace(k, dsu.get(v[0]));
        }

        /*cout << "step\n";
        for (auto [k, v]: asse)
            cout << k << ' ' << v << '\n';
        cout << '\n';*/

        return make_pair(s, asse.size());
    };

    int t, k, u;
    vector <Query> queries;
    for (int i = 0; i < q; i++) {
        cin >> t >> k;
        if (t == 3) {
            cin >> u;
            queries.emplace_back(i, t, k, u - 1);
        } else {
            queries.emplace_back(i, t, k);
        }
    }

    sort(queries.begin(), queries.end(), [] (Query lhs, Query rhs) {
        return lhs.k < rhs.k;
    });

    vector <ll> answers(q);

    int kx = 0; ll s = 0, d = 0;

    set <ll> seen;
    for (ll x: exp) seen.insert(x);
    d = seen.size();

    for (auto &query: queries) {
        while (kx < query.k) {
            if (d == 1) {
                kx++;
                continue;
            }

            auto [asum, diffs] = step();
            s += asum;
            d = diffs;
            kx++;
        }

        if (query.t == 3) {
            answers[query.index] = exp[dsu.get(query.i)];
        } else if (query.t == 1) {
            answers[query.index] = d;
        } else {
            answers[query.index] = s;
        }
    }

    for (ll x: answers) cout << x << '\n';
}

signed main() {
    cin.tie(NULL);
    ios_base::sync_with_stdio(false);

    #ifdef local
    freopen("input.txt", "r", stdin);
    #endif // local

    solve();
}
