#include <bits/stdc++.h>

using namespace std;
using ll = long long;

namespace Dinic {
    constexpr int MAXV = 1200;
    constexpr int MAXE = 2 * MAXV * MAXV;
    constexpr ll INF = (ll)1e18;

    struct edge {
        int to;
        ll f;
        ll c;

        edge() : to(-1), f(0), c(0) {};
        edge(int to, ll c) : to(to), f(0), c(c) {};
    } edg[MAXE];
    vector<int> g[MAXV];
    size_t ind[MAXV];
    int vert;
    ll max_cap;
    int dist[MAXV];
    int write_pos = 0;

    void add_edge(int v, int to, ll c) {
        g[v].push_back(write_pos);
        edg[write_pos++] = edge(to, c);
        g[to].push_back(write_pos);
        edg[write_pos++] = edge(v, 0);
        max_cap = max(max_cap, c);
    }

    bool bfs(int s, int t, ll scale) {
        fill(dist, dist + vert, MAXV);
        dist[s] = 0;
        queue<int> q;
        q.push(s);

        while (!q.empty()) {
            int v = q.front();
            q.pop();

            for (int e : g[v]) {
                int to = edg[e].to;
                if (edg[e].c - edg[e].f < scale) {
                    continue;
                }

                if (dist[to] > dist[v] + 1) {
                    dist[to] = dist[v] + 1;
                    q.push(to);
                }
            }
        }

        return dist[t] < MAXV;
    }

    ll dfs(int v, int t, ll scale, ll delta = INF) {
        if (v == t) {
            return delta;
        }

        for (; ind[v] < g[v].size(); ++ind[v]) {
            int e = g[v][ind[v]];
            int to = edg[e].to;
            if (dist[to] != dist[v] + 1 || edg[e].c - edg[e].f < scale) {
                continue;
            }

            ll delta1 = dfs(to, t, scale, min(delta, edg[e].c - edg[e].f));
            if (delta1 > 0) {
                edg[e].f += delta;
                edg[e ^ 1].f -= delta;
                return delta;
            }
        }

        return 0;
    }

    ll max_flow(int s, int t) {
        ll flow = 0;
        for (ll scale = max_cap; scale > 0; scale /= 2) {
            while (bfs(s, t, scale)) {
                ll pushed = 0;
                fill(ind, ind + vert, 0);
                while (pushed = dfs(s, t, scale)) {
                    flow += pushed;
                }
            }
        }
        return flow;
    }

    void clear_graph() {
        for (int v = 0; v < vert; ++v) {
            g[v].clear();
        }
        write_pos = 0;
    }
}

void solve() {
    int n, m;
    cin >> n >> m;
    vector<int> a(n);
    for (int i = 0; i < n; ++i) {
        cin >> a[i];
    }

    vector<int> lb(m), rb(m), c(m), t(m);
    for (int i = 0; i < m; ++i) {
        cin >> lb[i] >> rb[i] >> c[i] >> t[i];
        --lb[i], --rb[i];
    }

    for (int i = 0; i < n; ++i) {
        Dinic::vert = n + m + 2;
        int source = n + m;
        int terminal = n + m + 1;

        for (int v = 0; v < m; ++v) {
            Dinic::add_edge(source, v, c[v]);
            int l = lb[v], r = rb[v];
            if (t[v]) {
                l = min(l, i);
                r = max(r, i);
            }

            for (int to = l; to <= r; ++to) {
                Dinic::add_edge(v, m + to, c[v]);
            }
        }

        for (int v = 0; v < n; ++v) {
            Dinic::add_edge(m + v, terminal, a[v]);
        }

        cout << Dinic::max_flow(source, terminal) << ' ';
        Dinic::clear_graph();
    }
    cout << '\n';
}

int main() {
    ios::sync_with_stdio(0);
    cin.tie(0), cout.tie(0);

    int t;
    cin >> t;
    while (t--) {
        solve();
    }

    return 0;
}