#ifndef LOCAL
#pragma GCC optimize("Ofast")
#pragma GCC optimize("O3")
#endif

#include <iostream>
#include <vector>
#include <list>
#include <string>
#include <algorithm>
#include <queue>

#define len(a) (int)a.size()

typedef long long ll;


const int INF = (int)1e9;

using namespace std;

void run();

signed main()
{
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);
#endif

    run();

}

const char TYPE_UNITE = 1;
const char TYPE_FIRST = 2;
const char TYPE_SECOND = 3;
const char TYPE_THIRD = 4;

struct Event
{
    ll time;
    char type;
    int request_id;
    int ind;

    Event() : Event(-1, 127) {}
    Event(ll time, char type) : time(time), type(type) {}

    bool operator<(const Event& other) const
    {
        return tie(time, type) < tie(other.time, other.type);
    }
};

vector< vector<int> > graph;
vector< vector<int> > dp;
vector< vector<char> > reachable;

void dfs_reachable(int v, int forbidden)
{
    reachable[forbidden][v] = 1;
    for (int to : graph[v])
    {
        if (reachable[forbidden][to] || (to == forbidden))
        {
            continue;
        }
        dfs_reachable(to, forbidden);
    }
}

void solve_test()
{
    int n, m;
    cin >> n >> m;
    graph.assign(n, vector<int>());
    for (int i = 0; i < m; ++i)
    {
        int u, v;
        cin >> u >> v;
        --u, --v;
        graph[u].push_back(v);
        graph[v].push_back(u);
    }

    dp.assign(n, vector<int>(2, INF));
    dp[0][0] = 0;

    reachable.assign(n, vector<char>(n));
    for (int i = 0; i < n; ++i)
    {
        dfs_reachable(0, i);
    }

    queue< pair<int, int> > q;
    q.push({0, 0});
    while (!q.empty())
    {
        auto [v, s] = q.front();
        q.pop();

        if (s == 0)
        {
            for (int to : graph[v])
            {
                if (dp[v][0] + 1 < dp[to][1])
                {
                    dp[to][1] = dp[v][0] + 1;
                    q.push({to, 1});
                }
            }
        }
        else
        {
            for (int to : graph[v])
            {
                if (!reachable[v][to])
                {
                    continue;
                }
                if (dp[v][1] + 1 < dp[to][0])
                {
                    dp[to][0] = dp[v][1] + 1;
                    q.push({to, 0});
                }
            }
        }
    }

    for (int i = 1; i < n; ++i)
    {
        cout << (dp[i][0] == INF ? -1 : dp[i][0]) << ' ';
    }
    cout << '\n';

}

void run()
{
    int t;
    cin >> t;
    for (; t > 0; --t)
    {
        solve_test();
    }
}