#ifndef LOCAL
#pragma GCC optimize("Ofast")
#pragma GCC optimize("O3")
#endif

#include <iostream>
#include <vector>
#include <list>
#include <string>
#include <algorithm>
#include <queue>

#define len(a) (int)a.size()

typedef long long ll;


const int INF = (int)1e9;

using namespace std;

void run();

signed main()
{
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);
#endif

    run();

}

const char TYPE_UNITE = 1;
const char TYPE_FIRST = 2;
const char TYPE_SECOND = 3;
const char TYPE_THIRD = 4;

struct Event
{
    ll time;
    char type;
    int request_id;
    int ind;

    Event() : Event(-1, 127) {}
    Event(ll time, char type) : time(time), type(type) {}

    bool operator<(const Event& other) const
    {
        return tie(time, type) < tie(other.time, other.type);
    }
};

vector< vector<int> > graph;
vector< vector<int> > reachable;

void dfs_reachable(int v, int forbidden, int timer)
{
    reachable[forbidden][v] = timer;
    for (int to : graph[v])
    {
        if (reachable[forbidden][to] || (to == forbidden))
        {
            continue;
        }
        dfs_reachable(to, forbidden, timer);
    }
}

void solve_test()
{
    int n, m;
    cin >> n >> m;
    graph.assign(n, vector<int>());
    for (int i = 0; i < m; ++i)
    {
        int u, v;
        cin >> u >> v;
        --u, --v;
        graph[u].push_back(v);
        graph[v].push_back(u);
    }

    vector<int> with_box(n, INF);
    with_box[0] = 0;

    vector< vector< pair<int, int> > > placed(n);

    reachable.assign(n, vector<int>(n));
    for (int i = 0; i < n; ++i)
    {
        int timer = 1;
        for (int j = 0; j < n; ++j)
        {
            if (!reachable[i][j] && (j != i))
            {
                dfs_reachable(j, i, timer++);
            }

        }
    }

    queue< pair<int, int> > q;
    q.push({0, 0});
    while (!q.empty())
    {
        auto [v, s] = q.front();
        q.pop();

        if (v >= 0)
        {
            for (int to : graph[v])
            {
                placed[to].emplace_back(with_box[v] + 1, v);
                q.push({-(to + 1), len(placed[to]) - 1});
            }
        }
        else
        {
            v = -v - 1;
            auto [cost, from] = placed[v][s];
            for (int to : graph[v])
            {
                if (reachable[v][to] != reachable[v][from])
                {
                    continue;
                }
                if (cost + 1 < with_box[to])
                {
                    with_box[to] = cost + 1;
                    q.push({to, 0});
                }
            }
        }
    }

    for (int i = 1; i < n; ++i)
    {
        cout << (with_box[i] == INF ? -1 : with_box[i]) << ' ';
    }
    cout << '\n';

}

void run()
{
    int t;
    cin >> t;
    for (; t > 0; --t)
    {
        solve_test();
    }
}