#include <queue>
#include <sstream>
#include <functional>
#include <random>
#include <ctime>
#include <iostream>
#include <vector>
#include <set>
#include <algorithm>

using namespace std;

// #define int long long
constexpr int inf = 1e9 + 7;
using ll = long long;

const int MAXN = 5e5;
vector<int> G[MAXN];
int t_in[MAXN], t_out[MAXN];
int timer = 0;
int up[MAXN];

int k = 0;
vector<pair<int, int>> H[MAXN * 3];


int NewVertex() {
    return k++;
}


void SetupDFS(int v) {
    t_in[v] = timer++;
    for (int u : G[v]) {
        if (t_in[u] == -1) {
            SetupDFS(u);
        }
    }
    t_out[v] = timer++;
}

bool IsParent(int r, int v) {
    return t_in[r] <= t_in[v] && t_out[v] <= t_out[r];
}

bool used[MAXN];


void DFS(int v, int p = -1) {
    auto add_edge = [](int r, int b) {
        H[b].emplace_back(r, 1);
        H[r].emplace_back(b, 0);
    };

    up[v] = t_in[v];
    used[v] = true;

    int p_default = NewVertex();

    int last = -1, last_p = -1;
    for (int u : G[v]) {
        if (!used[u]) {
            DFS(u, v);
            up[v] = min(up[v], up[u]);
            if (up[u] >= t_in[v]) {
                last = u;
                last_p = NewVertex();
            }
        } else if (u != p) {
            up[v] = min(up[v], t_in[u]);
        }

        if (t_in[u] > t_in[v] && last != -1 && IsParent(last, u)) {
            add_edge(last_p, u);
        } else {
            add_edge(p_default, u);
        }
    }
}

void Setup(int n) {
    timer = 0;
    fill(G, G + n, vector<int>{});
    fill(t_in, t_in + n, -1);
    fill(t_out, t_out + n, -1);
    fill(H, H + n * 3, vector<pair<int, int>>{});
    k = n;
    fill(used, used + n, false);

}


void Solve(istream &in, ostream &out) {
    int t;
    in >> t;
    while (t--) {
        int n, m;
        in >> n >> m;
        Setup(n);
        for (int i = 0; i < m; ++i) {
            int a, b;
            in >> a >> b;
            --a;
            --b;
            G[a].push_back(b);
            G[b].push_back(a);
        }

        SetupDFS(0);

        for (int v = 0; v < n; ++v) {
            sort(G[v].begin(), G[v].end(),
                 [](int a, int b) {
                return t_in[a] < t_in[b];
            });
        }

        DFS(0);

        for (int v = 0; v < n * 2; ++v) {
            cerr << v + 1 << ":\n";
            for (auto [u, _] : H[v]) {
                cerr << u + 1 << ' ';
            }
            cerr << "\n\n";
        }

        vector<int> dist(n * 3, inf);
        dist[0] = 0;
        set<pair<int, int>> Q{ { 0, 0 } };
        while (!Q.empty()) {
            int v = Q.begin()->second;
            Q.erase(Q.begin());
            for (auto [u, w] : H[v]) {
                if (dist[v] + w < dist[u]) {
                    Q.erase({dist[u], u});
                    dist[u] = dist[v] + w;
                    Q.insert({dist[u], u});
                }
            }
        }

        for (int u = 1; u < n; ++u) {
            out << (dist[u] >= inf ? -1 : dist[u] * 2) << ' ';
        }
        out << '\n';
    }
}


//using Solution =

signed main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout.tie(0);
#endif
    Solve(cin, cout);
    return 0;
}
