#include <bits/stdc++.h>
using namespace std;

using ll = long long;
using ld = long double;

#define len(a) int((a).size())
#define all(a) begin(a), end(a)

template<typename Fun>
struct y_combinator {
    const Fun fun;

    y_combinator(const Fun&& fun) : fun(forward<const Fun>(fun)) {}

    template<typename... Args>
    decltype(auto) operator()(Args&&... args) const {
        return fun(ref(*this), forward<Args>(args)...);
    }
};

mt19937 rng(chrono::steady_clock::now().time_since_epoch().count());

struct node {
    node* left = nullptr, *right = nullptr;
    pair<int, int> x;
    ll value, sum_values = 0;
    int y;
    int sz;
    ll sum = 0, sum_segs;
    ll b = 0, k = 0;

    node(pair<int, int> x, ll value) : x(x), value(value), sum_values((x.second - x.first + 1) * value), y(rng()), sz(1), sum_segs(x.second - x.first + 1) {}

    ~node() {
        if (left) delete left;
        if (right) delete right;
    }
};

void apply(node* v, ll k, ll b) {
    assert(v);
    ll delta = b + (v->right ? v->right->sz : 0) * k;
    v->value += delta;
    v->sum_values += delta * (v->x.second - v->x.first + 1);
    v->k += k;
    v->b += b;
}

void push(node* v) {
    assert(v);
    if (v->right) apply(v->right, v->k, v->b);
    if (v->left) apply(v->left, v->k, v->b + v->k * (1 + (v->right ? v->right->sz : 0)));
    v->b = 0;
    v->k = 0;
}

void relax(node* v) {
    assert(v);
    assert(v->k == 0 && v->b == 0);
    v->sz = 1;
    if (v->left) v->sz += v->left->sz;
    if (v->right) v->sz += v->right->sz;

    v->sum_values = (v->x.second - v->x.first + 1) * v->value;
    if (v->left) v->sum_values += v->left->sum_values;
    if (v->right) v->sum_values += v->right->sum_values;

    v->sum_segs = v->x.second - v->x.first + 1;
    if (v->left) v->sum_segs += v->left->sum_segs;
    if (v->right) v->sum_segs += v->right->sum_segs;

    v->sum = 1ll * (v->right ? v->right->sz : 0) * (v->x.second - v->x.first + 1);
    if (v->right) v->sum += v->right->sum;
    if (v->left) v->sum += v->left->sum + (1 + (v->right ? v->right->sz : 0)) * v->left->sum_segs;
}

node* merge(node* left, node* right) {
    if (left) push(left);
    if (right) push(right);
    if (!left) return right;
    if (!right) return left;

    if (left->y > right->y) {
        left->right = merge(left->right, right);
        relax(left);
        return left;
    }
    right->left = merge(left, right->left);
    relax(right);
    return right;
}

pair<node*, node*> split_by_right(node* v, int value) {
    if (!v) return {v, v};
    push(v);
    if (v->x.second <= value) {
        auto [left, right] = split_by_right(v->right, value);
        v->right = left;
        relax(v);
        return {v, right};
    }
    auto [left, right] = split_by_right(v->left, value);
    v->left = right;
    relax(v);
    return {left, v};
}

pair<node*, node*> split_left(node* v) {
    assert(v);
    push(v);
    if (!v->left) {
        auto right = v->right;
        v->right = nullptr;
        relax(v);
        return {v, right};
    }
    auto [left, right] = split_left(v->left);
    v->left = right;
    relax(v);
    return {left, v};
}

ll solve(node* v, int pos, ll delta, int right = 0) {
    assert(v);
    push(v);
    if (v->x.first <= pos && pos <= v->x.second) {
        if (v->right) right += v->right->sz;
        return v->value + delta * right;
    }
    if (v->x.second < pos) return solve(v->right, pos, delta, right);
    right++;
    if (v->right) right += v->right->sz;
    return solve(v->left, pos, delta, right);
}

void print(node* v) {
    if (!v) return;
    push(v);
    print(v->left);
    cout << "[" << v->x.first << ", " << v->x.second << ", " << v->value << "] ";
    print(v->right);
}

int main() {
    ios::sync_with_stdio(false), cin.tie(nullptr);

    int n, q;
    cin >> n >> q;
    vector<ll> c(n);
    for (auto &x : c) cin >> x;

    struct query {
        int type;
        ll k;
        int i, id;
    };
    vector<query> qq(q);
    for (int i = 0; i < q; i++) qq[i].id = i;
    for (auto &[type, k, i, id] : qq) {
        cin >> type >> k;
        if (type == 3) cin >> i, i--;
    }
    sort(all(qq), [&](const query &a, const query &b) {
        return a.k < b.k;
    });

    vector<int> new_id(n);
    vector<int> ord(n);
    iota(all(ord), 0);
    sort(all(ord), [&](const int a, const int b) {
        return c[a] < c[b];
    });

    vector<ll> a(n);
    for (int i = 0; i < n; i++) {
        int j = ord[i];
        a[i] = c[j];
        new_id[j] = i;
    }

    node* root = nullptr;
    for (int i = 0; i < n; i++) {
        root = merge(root, new node({i, i}, a[i]));
    }

    vector<pair<ll, int>> events;
    for (int i = 0; i < n - 1; i++) {
        events.emplace_back(a[i + 1] - a[i], i);
    }
    events.emplace_back(1e17, -1);
    sort(all(events));

    int qpos = 0;
    ll cur_time = 0;
    vector<ll> ans(q);
    ll sum_c = accumulate(all(c), 0ll);

    // cout << "a: "; for (auto x : a) cout << x << ' '; cout << endl;
    // cout << "events: "; for (auto [a, b] : events) cout << "(" << a << ", " << b << ") "; cout << endl;
    for (int i = 0; i < n; i++) {
        // cout << "i = " << i << endl;
        // print(root); cout << endl << endl;

        while (qpos < q && qq[qpos].k < events[i].first) {
            auto [type, k, i, id] = qq[qpos];
            if (type == 1) {
                ans[id] = root->sz;
            } else if (type == 2) {
                ans[id] = root->sum_values;
                ans[id] += (k - cur_time) * root->sum;
                // for (int i = 0; i < n; i++) {
                //     ans[id] += solve(root, i, k - cur_time);
                // }
                ans[id] -= sum_c;
            } else {
                i = new_id[i];
                ans[id] = solve(root, i, k - cur_time);
            }
            qpos++;
        }
        if (i == n - 1) break;

        ll delta = events[i].first - cur_time;
        cur_time = events[i].first;
        apply(root, delta, 0);
        // cout << "after apply: "; print(root); cout << endl << endl;

        int pos = events[i].second;
        // cout << "unite pos = " << pos << endl;
        auto [left, m1m2right] = split_by_right(root, pos - 1);

        auto [m1, m2right] = split_by_right(m1m2right, pos);
        auto [m2, right] = split_left(m2right);

        assert(m1);
        assert(m2);
        assert(m1->left == nullptr);
        assert(m1->right == nullptr);
        assert(m2->left == nullptr);
        assert(m2->right == nullptr);
        assert(m1->value == m2->value);

        root = merge(left, merge(new node({m1->x.first, m2->x.second}, m1->value), right));
        delete m1;
        delete m2;
    }

    for (auto x : ans) cout << x << '\n';
    delete root;
}
