#pragma GCC optimize("Ofast")
//#pragma GCC target("avx2,sse4,popcnt")

#include <bits/stdc++.h>

#define sq(x) ((x)*(x))
#define err(x) cerr << "["#x"]  " << (x) << '\n'
#define errv(x) {cerr << "["#x"]  ["; for(const auto& i : (x))  cerr << i << ", "; cerr << "]\n";}
#define errvn(x, n) {cerr << "["#x"]  ["; for (int i = 0; i < (n); ++i) cerr << (x)[i] << ", "; cerr << "]\n";}
#define errf() cerr << "\n\n"

using namespace std;

using ll = long long;
using ld = long double;
using ull = unsigned long long;

mt19937 rng(chrono::steady_clock::now().time_since_epoch().count());
inline ll getrnd(ll l, ll r) {
    return uniform_int_distribution<ll>(l, r)(rng);
}

template<typename T1, typename T2>
inline bool relax(T1& a, const T2& b) {
    if (a > b) {
        a = b;
        return true;
    }
    return false;
}

template<typename T1, typename T2>
inline bool strain(T1& a, const T2& b) {
    if (a < b) {
        a = b;
        return true;
    }
    return false;
}

// :::::::::::::::::::::::::::::::::::

const ll INF = 1e18;

struct Flow {
    struct edge {
        ll to, c, f;
    };
    int n;
    vector<ll> d, ptr;
    vector<edge> edges;
    vector<vector<int>> graph;

    void create(int k) {
        n = k;
        graph.resize(n);
        d.resize(n);
        ptr.resize(n);
    }

    void add_edge(int x, int y, ll c) {
        int ret = edges.size();
        edges.push_back({y, c, 0});
        edges.push_back({x, 0, 0});
        graph[x].emplace_back(ret);
        graph[y].emplace_back(ret + 1);
    }

    bool bfs() {
        queue<int> scope;
        scope.push(0);
        d.assign(n, INF);
        d[0] = 0;
        while (!scope.empty()) {
            int v = scope.front();
            scope.pop();
            for (int ei : graph[v]) {
                auto& e = edges[ei];
                if (d[e.to] == INF && e.f < e.c) {
                    d[e.to] = d[v] + 1;
                    scope.emplace(e.to);
                }
            }
        }
        return d[n - 1] < INF;
    }

    ll dfs(int v, ll now) {
        if (v == n - 1 || now == 0) return now;
        while (ptr[v] < graph[v].size()) {
            int ei = graph[v][ptr[v]];
            auto& e = edges[ei];
            if (d[e.to] == d[v] + 1) {
                ll ret = dfs(e.to, min(now, e.c - e.f));
                if (ret != 0) {
                    e.f += ret;
                    edges[ei ^ 1].f -= ret;
                    return ret;
                }
            }
            ++ptr[v];
        }
        return 0;
    }

    ll max_flow() {
        ll ans = 0;
        while (bfs()) {
            ptr.assign(n, 0);
            while (ll ret = dfs(0, INF))
                ans += ret;
        }
        return ans;
    }
};

void solve() {
    int n, m;
    cin >> n >> m;
    vector<int> a(n);
    for (int i = 0; i < n; ++i)
        cin >> a[i];
    vector<tuple<int, int, int, int>> qs;
    for (int i = 0; i < m; ++i) {
        int l, r, c, t;
        cin >> l >> r >> c >> t;
        --l; --r;
        qs.emplace_back(l, r, c, t);
    }

    for (int i = 0; i < n; ++i) {
        Flow net;
        net.create(n + m + 2);
        int id = 0;
        for (auto [l, r, c, t] : qs) {
            if (t == 1) {
                relax(l, i);
                strain(r, i);
            }
            int now = ++id;
            net.add_edge(0, now, c);
            for (int i = l; i <= r; ++i)
                net.add_edge(now, m + 1 + i, a[i]);
        }
        for (int i = 0; i < n; ++i)
            net.add_edge(m + 1 + i, n + m + 1, a[i]);
        cout << net.max_flow() << ' ';
    }
    cout << '\n';
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr); cout.tie(nullptr);

    int t = 1;
    cin >> t;
    while (t--)
        solve();
    return 0;
}
