#include <iostream>
#include <algorithm>
#include <map>
#include <random>

std::vector<int64_t>* segtree;
int64_t** segtree_pref_sums;

void build(int32_t node, int32_t left, int32_t right, int64_t* arr) {
    segtree[node].resize(right - left);
    segtree_pref_sums[node] = new int64_t[right - left + 1];
    segtree_pref_sums[node][0] = 0;
    if(right - left == 1) {
        segtree[node][0] = arr[left];
        segtree_pref_sums[node][1] = arr[left];
        return;
    }
    int32_t m = (left + right) / 2;
    build(node * 2 + 1, left, m, arr);
    build(node * 2 + 2, m, right, arr);
    std::merge(segtree[node * 2 + 1].begin(), segtree[node * 2 + 1].end(),
               segtree[node * 2 + 2].begin(), segtree[node * 2 + 2].end(),
               segtree[node].begin());
    for(int32_t i = 1; i <= right - left; i++)
        segtree_pref_sums[node][i] = segtree_pref_sums[node][i - 1] + segtree[node][i - 1];
}

std::pair<int32_t, int64_t> query(int32_t node, int32_t left, int32_t right, int32_t query_left, int32_t query_right, int64_t k) {
    if(left >= query_right || right <= query_left)
        return {0, 0};
    if(left >= query_left && right <= query_right) {
        int32_t num = std::lower_bound(segtree[node].begin(), segtree[node].end(), k) - segtree[node].begin();
        return {num, segtree_pref_sums[node][num]};
    }
    int32_t m = (left + right) / 2;
    std::pair<int32_t, int64_t> res1 = query(node * 2 + 1, left, m, query_left, query_right, k);
    std::pair<int32_t, int64_t> res2 = query(node * 2 + 2, m, right, query_left, query_right, k);
    return {res1.first + res2.first, res1.second + res2.second};
}

int main() {
    //std::ios::sync_with_stdio(false);
    //std::cin.tie(0);

    int32_t n, num_q;
    std::cin >> n >> num_q;

    int64_t* arr = new int64_t[n];
    std::mt19937 rng;
    for(int32_t i = 0; i < n; i++)
        std::cin >> arr[i];
    //arr[i] = rng() % 500 + 1;

    int64_t* sorted_arr = new int64_t[n];
    for(int32_t i = 0; i < n; i++)
        sorted_arr[i] = arr[i];
    std::sort(sorted_arr, sorted_arr + n);

    int64_t* arr2 = new int64_t[n];
    for(int32_t i = 0; i < n; i++)
        arr2[i] = arr[i];
    std::sort(arr2, arr2 + n);
    int32_t levels_start = std::unique(arr2, arr2 + n) - arr2;

    std::vector<int64_t> diffs;
    std::map<int64_t, std::vector<int64_t> > per_diff;
    for(int32_t i = 0; i < levels_start - 1; i++) {
        diffs.push_back(arr2[i + 1] - arr2[i]);
        per_diff[diffs.back()].push_back(arr2[i]);
    }
    std::vector<int64_t> diffs2 = diffs;
    std::sort(diffs.begin(), diffs.end());

    std::vector<std::pair<int64_t, int64_t> > changes;
    changes.emplace_back(0, 0);
    for(int32_t i = 0; i < n; i++)
        changes[0].second += levels_start - (std::lower_bound(arr2, arr2 + levels_start, arr[i]) - arr2) - 1;

    for(int32_t i = 0; i < diffs.size(); i++) {
        if(i != diffs.size() - 1 && diffs[i + 1] == diffs[i])
            continue;

        //start to i inc
        changes.emplace_back(diffs[i], changes.back().second);
        for(int64_t num : per_diff[diffs[i]])
            changes.back().second -= std::lower_bound(sorted_arr, sorted_arr + n, num + 1) - sorted_arr;
    }

    int64_t* change_pref_sum = new int64_t[changes.size()];
    change_pref_sum[0] = 0;
    for(int32_t i = 1; i < changes.size(); i++)
        change_pref_sum[i] = change_pref_sum[i - 1] + (changes[i].first - changes[i - 1].first) * changes[i - 1].second;

    segtree = new std::vector<int64_t>[4 * diffs.size()];
    segtree_pref_sums = new int64_t*[4 * diffs.size()];
    build(0, 0, diffs.size(), diffs2.data());

    for(int32_t q = 0; q < num_q; q++) {
        int32_t type;
        int64_t k;
        std::cin >> type >> k;
        //type = 2;
        //k = rng() % 60;

        if(type == 1) {
            int32_t ans = std::lower_bound(diffs.begin(), diffs.end(), k + 1) - diffs.begin();
            ans = levels_start - ans;
            std::cout << ans << "\n";
        }
        if(type == 2) {
            int32_t period_in = std::lower_bound(changes.begin(), changes.end(), std::pair<int64_t, int64_t>(k, INT64_MAX)) - changes.begin() - 1;
            int64_t anss = 0;
            if(period_in == changes.size()) {
                std::cout << change_pref_sum[changes.size() - 1] << "\n";
                anss = change_pref_sum[changes.size() - 1];
            } else {
                std::cout << change_pref_sum[period_in] + (k - changes[period_in].first) * changes[period_in].second << "\n";
                anss = change_pref_sum[period_in] + (k - changes[period_in].first) * changes[period_in].second;
            }

            /*int64_t* arr_cur = new int64_t[n];
            for(int32_t i = 0; i < n; i++)
                arr_cur[i] = arr[i];
            int64_t* arr_cur2 = new int64_t[n];
            for(int32_t i = 0; i < k; i++) {
                for(int32_t j = 0; j < n; j++)
                    arr_cur2[j] = arr_cur[j];
                std::sort(arr_cur2, arr_cur2 + n);
                int32_t u = std::unique(arr_cur2, arr_cur2 + n) - arr_cur2;

                for(int32_t j = 0; j < n; j++)
                    arr_cur[j] += u - 1 - (std::lower_bound(arr_cur2, arr_cur2 + u, arr_cur[j]) - arr_cur2);
            }

            int64_t ans2 = 0;
            for(int32_t i = 0; i < n; i++)
                ans2 += arr_cur[i] - arr[i];
            int a = 0;
            if(ans2 != anss)
                a++;*/
        }
        if(type == 3) {
            int32_t ind;
            std::cin >> ind;
            ind--;

            int32_t start_go = levels_start - (std::lower_bound(arr2, arr2 + levels_start, arr[ind]) - arr2) - 1;
            int64_t ans = arr[ind] + k * start_go;

            int32_t ind_in_arr2 = std::lower_bound(sorted_arr, sorted_arr + n, arr[ind]) - sorted_arr;
            int32_t suf = diffs.size() - ind_in_arr2;
            std::pair<int32_t, int64_t> diff_small = query(0, 0, diffs.size(), diffs.size() - suf, diffs.size(), k);
            ans -= k * diff_small.first;
            ans += diff_small.second;
            /*for(int32_t i = ind_in_arr2; i < levels_start - 1; i++)
                if(arr2[i + 1] - arr2[i] < k)
                    ans -= k - (arr2[i + 1] - arr2[i]);*/

            std::cout << ans << "\n";
        }
    }
    return 0;
}
