#include <iostream>
#include <algorithm>
#include <map>
#include <random>
#include <bitset>

#pragma GCC target("popcnt")

int32_t* tin, *low, *depth;
int32_t timer_ = 0;
void dfs(int32_t v, int32_t parent, std::vector<int32_t>* graph, int32_t depth_cur, std::vector<int32_t>* children) {
    tin[v] = timer_++;
    low[v] = tin[v];
    depth[v] = depth_cur;
    for(int32_t i = 0; i < graph[v].size(); i++)
        if(tin[graph[v][i]] == -1) {
            dfs(graph[v][i], v, graph, depth_cur + 1, children);
            low[v] = std::min(low[v], low[graph[v][i]]);
            children[v].push_back(graph[v][i]);
        } else if(graph[v][i] != parent && depth[graph[v][i]] < depth[v])
            low[v] = std::min(low[v], tin[graph[v][i]]);
}

int32_t** ancestors;

void build_ancestor(int32_t v, int32_t parent, std::vector<int32_t>* tree) {
    ancestors[v] = new int32_t[14];
    ancestors[v][0] = v;
    ancestors[v][1] = parent;
    for(int32_t i = 2; i < 14; i++)
        ancestors[v][i] = ancestors[v][i - 1] == -1 ? -1 : ancestors[ancestors[v][i - 1]][i - 1];

    for(int32_t i = 0; i < tree[v].size(); i++)
        build_ancestor(tree[v][i], v, tree);
}

int32_t kth_ancestor(int32_t v, int32_t k) {
    for(int32_t i = 13; i > 0; i--)
        if(k & (1 << (i - 1)))
            v = ancestors[v][i];
    return v;
}

int main() {
    std::ios::sync_with_stdio(false);
    std::cin.tie(0);

    int32_t num_tests;
    std::cin >> num_tests;

    std::mt19937 rng;
    for(int32_t t = 0 ;t < num_tests; t++) {
        int32_t n, m;
        std::cin >> n >> m;

        std::vector<int32_t>* graph = new std::vector<int32_t>[n];
        for(int32_t i = 0; i < m; i++) {
            int32_t v1, v2;
            std::cin >> v1 >> v2;
            v1--;
            v2--;
            /*v1 = rng() % n;
            v2 = rng() % n;
            while(v2 == v1)
                v2 = rng() % n;*/

            graph[v1].push_back(v2);
            graph[v2].push_back(v1);
        }

        tin = new int32_t[n];
        depth = new int32_t[n];
        low = new int32_t[n];
        for(int32_t i = 0; i < n; i++)
            tin[i] = -1;
        std::vector<int32_t>* tree = new std::vector<int32_t>[n];
        dfs(0, -1, graph, 0, tree);

        ancestors = new int32_t*[n];
        build_ancestor(0, -1, tree);

        std::bitset<5000>* neighbors = new std::bitset<5000>[n];
        for(int32_t i = 0; i < n; i++)
            for(int32_t j = 0; j < graph[i].size(); j++)
                neighbors[i][graph[i][j]] = true;

        int32_t* min_from = new int32_t[n];
        bool* used = new bool[n];
        for(int32_t i = 0; i < n; i++) {
            used[i] = false;
            min_from[i] = INT32_MAX;
        }
        min_from[0] = 0;
        for(int32_t z = 0; z < n; z++) {
            int32_t v = -1;
            for(int32_t i = 0; i < n; i++)
                if(!used[i] && (v == -1 || min_from[i] < min_from[v]))
                    v = i;
            if(v == -1 || min_from[v] == INT32_MAX)
                break;
            used[v] = true;

            for(int32_t i = 0; i < n; i++) {
                if(used[i])
                    continue;

                std::bitset<5000> common = (neighbors[i] & neighbors[v]);
                int32_t common_neighb = common.count();

                if(common_neighb >= 1) {
                    int32_t deleted = common._Find_first();
                    bool up_i = depth[i] < depth[deleted];
                    int32_t repr1 = -1, repr2 = -2;
                    if(!up_i) {
                        repr1 = kth_ancestor(i, depth[i] - depth[deleted] - 1);
                        up_i |= low[repr1] < tin[deleted];
                    }
                    bool up_v = depth[v] < depth[deleted];
                    if(!up_v) {
                        repr2 = kth_ancestor(v, depth[v] - depth[deleted] - 1);
                        up_v |= low[repr2] < tin[deleted];
                    }

                    if(repr1 == repr2 || (up_i && up_v)) {
                        min_from[i] = std::min(min_from[i], min_from[v] + 2);
                    }
                }
            }
        }

        for(int32_t i = 1; i < n; i++) {
            std::cout << (min_from[i] == INT32_MAX ? -1 : min_from[i]) << " ";
        }
        std::cout << "\n";
    }
    return 0;
}