#include <iostream>
#include <algorithm>
#include <set>

struct Robot {
    int32_t left, right, cnt, type;
};

int32_t n;

int32_t* parents;
int32_t* comp_max;

int32_t get_root(int32_t v) {
    if(parents[v] == v)
        return v;
    return parents[v] = get_root(parents[v]);
}

void join(int32_t v1, int32_t v2) {
    v1 = get_root(v1);
    v2 = get_root(v2);
    if(v1 == v2)
        return;
    parents[v1] = v2;
    comp_max[v2] = std::max(comp_max[v2], comp_max[v1]);
}

int64_t solve( int32_t m, Robot* robots, int32_t* arr) {
    std::sort(robots, robots + m, [](Robot r1, Robot r2) {
        return r1.right < r2.right;
    });
    for(int32_t i = 0; i < n; i++) {
        parents[i] = i;
        comp_max[i] = i;
    }
    int64_t ans = 0;
    for(int32_t j = 0; j < m; j++) {
        Robot &r = robots[j];
        for(int32_t i = r.left; i <= r.right;) {
            int32_t take = std::min(arr[i], r.cnt);
            r.cnt -= take;
            arr[i] -= take;
            ans += take;

            if(arr[i] == 0) {
                if (i + 1 < n && arr[i + 1] == 0)
                    join(i, i + 1);
                if (i - 1 >= 0 && arr[i - 1] == 0)
                    join(i, i - 1);
            }
            if(r.cnt == 0)
                break;

            i++;
            if(i > r.right)
                break;
            if(arr[i] == 0) {
                i = comp_max[get_root(i)] + 1;
            }
        }
    }
    return ans;
}

int main() {
    int32_t num_tests;
    std::cin >> num_tests;

    for(int32_t t = 0 ;t < num_tests; t++) {
        int32_t m;
        std::cin >> n >> m;

        int32_t* arr = new int32_t[n];
        for(int32_t i = 0; i < n; i++)
            std::cin >> arr[i];

        Robot* robots = new Robot[m];
        for(int32_t i = 0; i < m; i++) {
            std::cin >> robots[i].left >> robots[i].right >> robots[i].cnt >> robots[i].type;
            robots[i].left--;
            robots[i].right--;
        }

        Robot* robot2 = new Robot[m];
        int32_t* arr2 = new int32_t[n];
        parents = new int32_t[n];
        comp_max = new int32_t[n];
        for(int32_t i = 0; i < n; i++) {
            for(int32_t j = 0; j < m; j++) {
                robot2[j] = robots[j];
                robot2[j].left = (robots[j].type == 0 ? robots[j].left : std::min(i, robots[j].left));
                robot2[j].right = (robots[j].type == 0 ? robots[j].right : std::max(i, robots[j].right));
            }
            for(int32_t j = 0; j < n; j++)
                arr2[j] = arr[j];
            std::cout << solve(m, robot2, arr2) << " ";
        }
        std::cout << "\n";
    }
    return 0;
}