#ifdef LOCAL
#define _GLIBCXX_DEBUG
#else
// #pragma GCC optimize("O3")
// #pragma GCC optimize("fast-math")
// #pragma GCC optimize("unroll-loops")
// #pragma GCC optimize("Ofast")
// #pragma GCC target("avx2,popcnt,lzcnt,bmi,bmi2")
#endif

#include <algorithm>
#include <cassert>
#include <chrono>
#include <cstdlib>
#include <cstring>
#include <deque>
#include <iomanip>
#include <iostream>
#include <map>
#include <numeric>
#include <queue>
#include <random>
#include <set>
#include <unordered_map>
#include <unordered_set>
#include <vector>

using namespace std;
using ll = long long;

const int INF = 0x3f3f3f3f;
const ll INFLL = 0x3f3f3f3f3f3f3f3fll;

struct DSU {
    int n;
    vector<int> p, siz, cnt;

    DSU(int n) : n(n), p(n), siz(n, 1) { iota(p.begin(), p.end(), 0); }

    int find(int v) {
        if (p[v] == v) return v;
        return p[v] = find(p[v]);
    }

    bool merge(int v, int u) {
        v = find(v), u = find(u);
        if (v == u) return false;
        if (siz[v] < siz[u]) swap(v, u);
        siz[v] += siz[u];
        p[u] = v;
        return true;
    }
};

struct Graph {
    int n, cnt;
    vector<vector<pair<int, int>>> g;
    DSU col;
    vector<int> vis, fup, h;

    Graph(int n, int m)
        : n(n), cnt(m), g(n), col(m), vis(n, 0), fup(n, INF), h(n, 0) {}

    void add_edge(int v, int u, int i) {
        g[v].emplace_back(u, i);
        g[u].emplace_back(v, i);
    }

    void dfs(int v, int p, int pi) {
        vis[v] = 1;
        for (auto [u, i] : g[v]) {
            if (u == p) continue;
            if (vis[u] == 1) {
                if (col.merge(i, pi)) cnt--;
                fup[v] = min(fup[v], h[u]);
                continue;
            }
            if (vis[u] == 0) {
                h[u] = h[v] + 1;
                dfs(u, v, i);
                fup[v] = min(fup[v], fup[u]);
                if (fup[u] < h[v]) {
                    if (col.merge(i, pi)) cnt--;
                }
            }
        }
        vis[v] = 2;
    }
};

void solve() {
    int n, m;
    cin >> n >> m;
    Graph g(n, m);
    for (int i = 0; i < m; i++) {
        int v, u;
        cin >> v >> u;
        v--, u--;
        g.add_edge(v, u, i);
    }
    g.dfs(0, -1, -1);
    vector<int> ans(n, INF);
    ans[0] = 0;
    deque<int> q;
    q.push_back(0);
    while (!q.empty()) {
        auto v = q.front();
        q.pop_front();
        for (auto [u, i] : g.g[v]) {
            for (auto [z, j] : g.g[u]) {
                if (ans[z] != INF) continue;
                if (g.col.find(i) != g.col.find(j)) continue;
                ans[z] = ans[v] + 2;
                q.push_back(z);
            }
        }
    }
    for (int i = 1; i < n; i++) {
        if (ans[i] == INF) {
            cout << "-1 ";
        } else {
            cout << ans[i] << ' ';
        }
    }
    cout << '\n';
}

signed main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif

    int t = 1;
    cin >> t;
    while (t--) solve();
}