#include <iostream>
#include <algorithm>
#include <vector>
#include <cmath>
#include <numeric>
#include <string>
#include <map>
#include <set>

#define all(x) x.begin(), x.end()

using namespace std;
using ll = long long;

struct query
{
	int type, player, ind;
	ll time;

	query(int type, ll time, int ind, int player = 0) : type(type), player(player), ind(ind), time(time) {}
	bool operator<(const query& q) const
	{
		return time < q.time;
	}
};

const int MAXN = 300'000;

int p[MAXN];
int rnk[MAXN];
int sz[MAXN];

int getComp(int u)
{
	if (p[u] == u)
		return u;
	return p[u] = getComp(p[u]);
}

int merge(int u, int v)
{
	u = getComp(u), v = getComp(v);
	if (u == v)
		return u;
	//if (rnk[u] < rnk[v])
	//	swap(u, v);
	p[v] = u;
	//if (rnk[v] == rnk[u])
	//	++rnk[u];
	sz[u] += sz[v];
	return u;
}

struct SegTree
{
	vector<ll> sums;

	SegTree(int n)
	{
		sums.resize(4 * n);
	}

	ll get(int i, int l, int r, int ql, int qr)
	{
		if (l >= qr || r <= ql)
			return 0;
		if (ql <= l && r <= qr)
			return sums[i];

		int m = (l + r) / 2;
		return get(i * 2 + 1, l, m, ql, qr) + get(i * 2 + 2, m, r, ql, qr);
	}

	void update(int i, int l, int r, int pos, ll val)
	{
		if (pos < l || pos >= r)
			return;
		if (l + 1 == r)
		{
			sums[i] = val;
			return;
		}

		int m = (l + r) / 2;
		update(i * 2 + 1, l, m, pos, val);
		update(i * 2 + 2, m, r, pos, val);
		sums[i] = sums[i * 2 + 1] + sums[i * 2 + 2];
	}
};

int main()
{
	ios::sync_with_stdio(false);
	cin.tie(0), cout.tie(0);

	int  n = 0, q = 0;
	cin >> n >> q;

	vector<ll> c(n);
	for (int i = 0; i < n; ++i)
		cin >> c[i];

	vector<query> queries;
	int type = 0, player = 0;
	ll t = 0;
	for (int i = 0; i < q; ++i)
	{
		cin >> type;
		if (type == 1 || type == 2)
			cin >> t;
		else
			cin >> t >> player;
		queries.push_back(query(type, t, i, player));
	}

	sort(all(queries));
	vector<ll> ans(q, 1);

	iota(p, p + n, 0);
	fill(sz, sz + n, 1);

	map<ll, int> comp;
	for (int i = 0; i < n; ++i)
	{
		if (comp.count(c[i]))
			merge(i, comp[c[i]]);
		else
			comp[c[i]] = i;
	}

	set<pair<ll, int>> cs;
	for (int i = 0; i < n; ++i)
		if (p[i] == i)
			cs.emplace(c[i], i);

	set<pair<ll, int>> dist;
	for (auto it = cs.begin(); next(it) != cs.end(); ++it)
	{
		dist.emplace(next(it)->first - it->first, it->second);
	}

	vector<pair<ll, ll>> sizes(cs.size());
	int sizesPos = 0;
	for (auto x : cs)
	{
		sizes[sizesPos].second = sz[x.second];
		sizes[sizesPos].first = x.first;
		++sizesPos;
	}

	vector<pair<ll, ll>> prefSizes(cs.size() + 1);
	for (int i = 0; i < cs.size(); ++i)
		prefSizes[i + 1] = { sizes[i].first, sizes[i].second + prefSizes[i].second };

	ll curTime = 0;
	int qPos = 0;

	ll curSum = 0;
	for (int i = sizes.size() - 1; i >= 0; --i)
		curSum += sizes[i].second * (sizes.size() - 1 - i);

	ll totalAdd = 0;
	ll prevTime = 0;

	SegTree ones(cs.size()), sums(cs.size());

	while (!dist.empty())
	{
		ll nextTime = dist.begin()->first;
		while (qPos < q && queries[qPos].time < nextTime)
		{
			auto quer = queries[qPos];
			if (quer.type == 1)
			{
				ans[quer.ind] = dist.size() + 1;
			}
			else if (quer.type == 2)
			{
				ans[quer.ind] = totalAdd + curSum * (quer.time - prevTime);
			}
			else
			{
				int pos = upper_bound(all(prefSizes), make_pair((ll)c[quer.player - 1], 0ll)) - prefSizes.begin();
				ans[quer.ind] = sums.get(0, 0, cs.size(), pos, cs.size()) +
					quer.time * (cs.size() - pos - ones.get(0, 0, cs.size(), pos, cs.size())) + c[quer.player - 1];
			}
			++qPos;
		}
		totalAdd += (nextTime - prevTime) * curSum;

		auto [dif, gr] = *dist.begin();
		dist.erase(dist.begin());
		auto pos = upper_bound(all(prefSizes), make_pair((ll)c[gr], 0ll)) - prefSizes.begin();
		curSum -= prefSizes[pos].second;
		ones.update(0, 0, cs.size(), pos - 1, 1);
		sums.update(0, 0, cs.size(), pos - 1, nextTime);
		/*auto it = cs.find({ c[gr], gr });
		int u = next(it)->second;
		int top = merge(gr, u);*/
		prevTime = nextTime;
	}

	while (qPos < q)
	{
		auto quer = queries[qPos];
		if (quer.type == 1)
		{
			ans[quer.ind] = 1;
		}
		else if (quer.type == 2)
		{
			ans[quer.ind] = totalAdd + curSum * (quer.time - prevTime);
		}
		else
		{
			int pos = upper_bound(all(prefSizes), make_pair((ll)c[quer.player - 1], 0ll)) - prefSizes.begin();
			ans[quer.ind] = sums.get(0, 0, cs.size(), pos - 1, cs.size()) +
				quer.time * (cs.size() - pos - ones.get(0, 0, cs.size(), pos, cs.size())) + c[quer.player - 1];
		}
		++qPos;
	}

	for (auto x : ans)
		cout << x << "\n";

	return 0;
}