#include <iostream>
#include <string>
#include <vector>
#include <algorithm>
#include <unordered_map>
#include <set>

typedef long long ll;

using namespace std;

struct Group {
    int id;
    ll base;
    ll delta;
    int count;
};

struct Request {
    int type;
    ll time;
    int idx = -1;
};

struct Event {
    ll time;
    int type;
    int idx, idx2;
};

int collect_group(vector<Group> &groups, int idx, bool first = true) {
    if (groups.at(idx).id != idx) {
        int new_idx = groups.at(idx).id;

        groups.at(new_idx).count += groups.at(idx).count;
        groups.at(idx).count = 0;

        int parent = collect_group(groups, new_idx, false);
        if (first) for (int i = idx; i < parent; ++i) groups[i].id = parent;

        return parent;

    }
    return idx;
}

int main() {
    int n, q;
    cin >> n >> q;

    vector<ll> cs(n);
    for (auto &c: cs) cin >> c;

    vector<ll> sorted_cs = cs;
    sort(sorted_cs.begin(), sorted_cs.end());

    vector<Group> groups(n);
    groups[n - 1] = {n - 1, sorted_cs.at(n - 1), 0, 1};

    vector<Event> events;

    ll exp_per_tick = 0;

    int current_group_idx = n - 1;

    ll group_count = 1;

    set<int> group_ids;
    group_ids.insert(n - 1);

    for (int i = n - 2; i >= 0; --i) {
        if (sorted_cs.at(i) == sorted_cs.at(i + 1)) {
            groups[current_group_idx].count += 1;
            groups[i].id = current_group_idx;

            exp_per_tick += group_count - 1;
        } else {
            groups[i] = {i, sorted_cs.at(i), sorted_cs.at(i + 1) - sorted_cs.at(i), 1};

            group_ids.insert(i);

            current_group_idx = i;

            events.push_back({groups[i].delta, 0, groups[i].id});

            exp_per_tick += group_count;
            group_count += 1;
        }
    }

    vector<int> sorted_idx_by_idx(n);
    for (int i = 0; i < n; ++i) {
        sorted_idx_by_idx[i] = lower_bound(sorted_cs.begin(), sorted_cs.end(), cs[i]) - sorted_cs.begin();
    }

    sorted_cs.clear();

    vector<ll> ans(q);

    vector<Request> requests(q);
    for (int i = 0; i < q; ++i) {
        auto &req = requests.at(i);

        cin >> req.type >> req.time;
        if (req.type == 3) {
            cin >> req.idx;
            req.type = -1;
        }
        events.push_back({req.time, req.type, i, req.idx - 1});
    }

    sort(events.begin(), events.end(), [](const Event &left, const Event &right) -> bool {
        if (left.time < right.time) return true;
        else if (left.time > right.time) return false;
        else return left.type < right.type;
    });

    ll prev_time = 0;
    ll d_exp = 0;

    for (const auto &event: events) {
        d_exp += exp_per_tick * (event.time - prev_time);

        if (event.type == 0) {
            int group_id = event.idx;

            group_ids.erase(group_id);

            exp_per_tick -= groups[group_id].count;

            auto g = groups[group_id];

            int lower;
            do {
                auto l = lower_bound(groups.begin(), groups.end(), g,
                                    [](const Group &left, const Group &right) {
                                        return left.id < right.id;
                                    });
                lower = l - groups.begin();
                collect_group(groups, lower);
            } while (groups[lower].id != lower);

            groups.at(group_id).id = groups.at(group_id + 1).id;
            collect_group(groups, group_id);

            group_count -= 1;

            exp_per_tick -= lower;
        } else if (event.type == 1) {
            ans.at(event.idx) = group_count;
        } else if (event.type == 2) {
            ans.at(event.idx) = d_exp;
        } else {
            int group_id = sorted_idx_by_idx[event.idx2];
            collect_group(groups, group_id);

            group_id = groups[group_id].id;

            auto itr = group_ids.find(group_id);

            ans.at(event.idx) = 0;
            while (itr != group_ids.end()) {
                ++itr;
                ans.at(event.idx) += 1;
            }

            ans[event.idx] -= 1;

            ans.at(event.idx) = ans.at(event.idx) * event.time + groups.at(group_id).base;
        }

        prev_time = event.time;
    }

    for (auto a: ans) {
        cout << a << "\n";
    }

    return 0;
}