#include <bits/stdc++.h>

struct dsu {
    std::vector<int> par, add_info;
    dsu() = default;
    dsu(int n) : par(n, -1) {
        add_info.assign(n, 0);
        std::iota(add_info.begin(), add_info.end(), 0);
    }
    int get(int v) {
        return par[v] < 0 ? v : par[v] = get(par[v]);
    }
    void unite(int p, int v) {
        p = get(p);
        v = get(v);
        int predstav = add_info[p];
        if (p == v)
            assert(0); // what
        if (-par[p] < -par[v])
            std::swap(p, v);
        par[p] += par[v];
        par[v] = p;
        add_info[p] = predstav;
    }
};

struct fenwick {
    int n;
    std::vector<int64_t> t;
    fenwick() = default;
    fenwick(int n) : n(n), t(n + 1) {}
    void add(int pos, int64_t x) {
        for (; pos <= n; pos += pos & -pos)
            t[pos] += x;
    }
    int64_t get(int pos) {
        int64_t res = 0;
        for (; pos > 0; pos -= pos & -pos)
            res += t[pos];
        return res;
    }
};

signed main() {
    std::ios::sync_with_stdio(false);
    std::cin.tie(nullptr);
    int n, q;
    std::cin >> n >> q;
    std::vector<int64_t> c(n);
    std::map<int64_t, int64_t> c_cnt;
    for (auto &x : c) {
        std::cin >> x;
        c_cnt[x]++;
    }
    std::vector<int64_t> c_srtd = c;
    std::sort(c_srtd.begin(), c_srtd.end());
    auto c_srtd_nu = c_srtd;
    c_srtd.erase(std::unique(c_srtd.begin(), c_srtd.end()), c_srtd.end());
    std::reverse(c_srtd.begin(), c_srtd.end());
    int n_s = c_srtd.size();
    std::vector<int64_t> moments;
    std::vector<std::tuple<int64_t, int64_t, int64_t>> moments_cnts; // [{moment, how_many, speed}]
    int64_t summary_speed = 0;
    for (int i = 1; i < n_s; ++i) {
        moments.emplace_back(c_srtd[i - 1] - c_srtd[i]);
        moments_cnts.emplace_back(c_srtd[i - 1] - c_srtd[i], c_cnt[c_srtd[i]], i);
        summary_speed += c_cnt[c_srtd[i]] * i;
    }

    const int64_t INF = 1e13;
    moments.emplace_back(INF);
    moments_cnts.emplace_back(INF, c_cnt[c_srtd[0]], 0);
    std::sort(moments.begin(), moments.end());
    std::sort(moments_cnts.begin(), moments_cnts.end());

    std::vector<std::pair<int64_t, int64_t>> moments_speeds;
    moments_speeds.emplace_back(0, summary_speed);
    for (int i = 0; i + 1 < n_s; ++i) {
        summary_speed -= std::upper_bound(c_srtd_nu.begin(), c_srtd_nu.end(), c_srtd[std::get<2>(moments_cnts[i])]) -
                c_srtd_nu.begin();
        moments_speeds.emplace_back(moments[i], summary_speed);
    }
    std::vector<int64_t> pref_sums(n_s);
    for (int i = 1; i < n_s; ++i) {
        pref_sums[i] = pref_sums[i - 1] + moments_speeds[i - 1].second * (moments_speeds[i].first - moments_speeds[i - 1].first);
    }
    std::vector<int64_t> ans(q);
    std::vector<std::tuple<int64_t, int64_t, int64_t>> queries;
    std::reverse(c_srtd.begin(), c_srtd.end());
    for (int ind_q = 0; ind_q < q; ++ind_q) {
        int t;
        std::cin >> t;
        if (t == 1) {
            int64_t k;
            std::cin >> k;
            int pos = std::upper_bound(moments.begin(), moments.end(), k) - moments.begin();
            ans[ind_q] = n_s - pos;
//            std::cout << n_s - pos << "\n";
        } else if (t == 2) {
            int64_t k;
            std::cin >> k;
            int pos = std::upper_bound(moments.begin(), moments.end(), k) - moments.begin();
            ans[ind_q] = pref_sums[pos] + moments_speeds[pos].second * (k - moments_speeds[pos].first);
//            std::cout << pref_sums[pos] + moments_speeds[pos].second * (k - moments_speeds[pos].first) << "\n";
        } else {
            int64_t k, i;
            std::cin >> k >> i, --i;
            queries.emplace_back(k, std::lower_bound(c_srtd.begin(), c_srtd.end(), c[i]) - c_srtd.begin(), ind_q);
        }
    }
    dsu points(n_s);
    std::sort(queries.begin(), queries.end());
    int ind_quer = 0, ind_mom = 0, qrs = queries.size();
    fenwick con(n_s), coef(n_s);
    for (int i = 0; i < n_s; ++i)
        con.add(i + 1, c_srtd[n_s - 1 - i]), con.add(i + 2, -c_srtd[n_s - 1 - i]), coef.add(i + 2, 1);
    while (ind_quer < qrs) {
        while (std::get<0>(queries[ind_quer]) >= moments[ind_mom]) {
            points.unite(n_s - 1 - (std::get<2>(moments_cnts[ind_mom]) - 1), n_s - 1 - std::get<2>(moments_cnts[ind_mom]));
            int col = std::get<2>(moments_cnts[ind_mom]);
            con.add(col + 1, moments[ind_mom]);
            coef.add(col + 1, -1);
            ++ind_mom;
        }
        int predstavitel = points.get(std::get<1>(queries[ind_quer]));
        predstavitel = n_s - 1 - predstavitel;
        ans[std::get<2>(queries[ind_quer])] = con.get(predstavitel + 1) + std::get<0>(queries[ind_quer]) * coef.get(predstavitel + 1);
        ++ind_quer;
    }
    for (auto &x : ans)
        std::cout << x << "\n";
    return 0;
}
