#include <iostream>
#include <string>
#include <algorithm>
#include <vector>
#include <map>
#include <tuple>
#include <numeric>
#include <queue>

using namespace std;

using ll = long long;
template <typename T>
using v = vector<T>;

template <typename T>
istream& operator>> (istream& in, v<T>& a) {
	for (T& x : a) {
		in >> x;
	}
	return in;
}

struct segment_tree {
	using tree_el = ll;
	using push_el = ll;
	ll n;
	v<tree_el> st;
	v<push_el> p;
	tree_el neutral;
	push_el p_neutral;
	v<ll> changed_st;
	v<ll> changed_p;

	segment_tree() = default;

	segment_tree(const v<tree_el>& a) : n(a.size()), st(4 * a.size()), neutral(), p_neutral() {
		p = v<push_el>(4 * a.size(), p_neutral);
		build(a, 0, 0, n);
	}

	tree_el combine_els(tree_el a, tree_el b) {
		return a + b;
	}

	tree_el build(const v<tree_el>& a, ll i, ll L, ll R) {
		if (R - L != 0) {
			if (R - L == 1) {
				return st[i] = a[L];
			}
			else {
				ll M = (L + R) >> 1;
				return st[i] = combine_els(build(a, (i << 1) + 1, L, M), build(a, (i << 1) + 2, M, R));
			}
		}
	}

	void push(ll i) {
		changed_st.push_back(i);
		st[i] -= p[i];
		ll a = (i << 1) + 1;
		if (a < p.size()) {
			ll s = min(st[a] - p[a], p[i]);
			changed_p.push_back(a);
			p[a] += s;
			ll b = (i << 1) + 2;
			if (b < p.size()) {
				changed_p.push_back(b);
				p[b] += p[i] - s;
			}
		}
		changed_p.push_back(i);
		p[i] = p_neutral;
	}

	tree_el get_range(ll inL, ll inR, ll i = 0, ll L = 0, ll R = -1) {
		if (R == -1) {
			R = n;
		}
		if (R - L != 0) {
			push(i);
			if (R <= inL || L >= inR) {
				return neutral;
			}
			else if (L >= inL && R <= inR) {
				return st[i];
			}
			else {
				ll M = (L + R) >> 1;
				return combine_els(get_range(inL, inR, (i << 1) + 1, L, M), get_range(inL, inR, (i << 1) + 2, M, R));
			}
		}
	}

	ll sub_range(ll inL, ll inR, push_el inX, ll i = 0, ll L = 0, ll R = -1) {
		if (R == -1) {
			R = n;
		}
		if (R - L != 0) {
			push(i);
			if (L >= inR || R <= inL) {
				return 0;
			}
			else if (L >= inL && R <= inR) {
				ll s = min(st[i], inX);
				changed_p.push_back(i);
				p[i] += s;
				push(i);
				return s;
			}
			else {
				ll M = (L + R) >> 1;
				ll a = sub_range(inL, inR, inX, (i << 1) + 1, L, M);
				if (inX == a) {
					changed_st.push_back(i);
					st[i] = combine_els(st[(i << 1) + 1], st[(i << 1) + 2]);
					return a;
				}
				else {
					ll b = sub_range(inL, inR, inX - a, (i << 1) + 2, M, R);
					changed_st.push_back(i);
					st[i] = combine_els(st[(i << 1) + 1], st[(i << 1) + 2]);
					return a + b;
				}
			}
		}
	}
};

void solve() {
	ll n, m;
	cin >> n >> m;
	v<ll> a(n);
	cin >> a;
	v<ll> l(m), r(m), c(m), t(m);
	for (ll i = 0; i < m; ++i) {
		cin >> l[i] >> r[i] >> c[i] >> t[i];
		--l[i];
		--r[i];
	}
	segment_tree s(a);
	segment_tree s2(a);
	for (ll x = 0; x < n; ++x) {
		v<ll> inds(m);
		iota(inds.begin(), inds.end(), 0ll);
		sort(inds.begin(), inds.end(), [&](ll ria, ll rib) {
			ll ar;
			if (t[ria]) {
				ar = max(r[ria], x);
			}
			else {
				ar = r[ria];
			}
			ll br;
			if (t[rib]) {
				br = max(r[rib], x);
			}
			else {
				br = r[rib];
			}
			return ar < br;
			});
		ll cres = 0;
		for (ll ri : inds) {
			ll cl, cr;
			if (t[ri]) {
				cl = min(l[ri], x);
				cr = max(r[ri], x);
			}
			else {
				cl = l[ri];
				cr = r[ri];
			}
			cres += s2.sub_range(cl, cr + 1, c[ri]);
		}
		cout << cres << ' ';
		for (ll x : s2.changed_st) {
			s2.st[x] = s.st[x];
		}
		s2.changed_st.clear();
		for (ll x : s2.changed_p) {
			s2.p[x] = s.p[x];
		}
		s2.changed_p.clear();
	}
}

int main() {
	ll t;
	cin >> t;
	for (ll i = 0; i < t; ++i) {
		solve();
	}
}