#include <iostream>
#include <vector>
#include <cassert>
#include <set>
#include <algorithm>
#include <map>
#include <random>

using namespace std;

#define int int64_t
#define bint __int128_t
#define fi first
#define se second
#define all(x) (x).begin(),(x).end()

mt19937 rnd(100);

struct node {
    int sz, y;
    int val, sum;
    int cnt;
    vector<int> inds;
    node *l, *r, *pr, *next;
    int st, del;
    node() {
        cnt = 0;
        sum = 0;
        st = 0;
        del = 0;
        y = rnd();
        sz = 1;
        l = r = pr = next = nullptr;
    }
};

int sz(node *v) {
    return (!v ? 0 : v->sz);
}
int cnt(node *v) {
    return (!v ? 0 : v->cnt);
}
int sum(node *v) {
    return (!v ? 0 : v->sum + (v->st + (cnt(v)-1) * v->del) * v->sz / 2);
}

void upd(node *v) {
    if (!v) return;
    //assert(v->st == 0 && v->del == 0);
    //push(v);
    v->sz = sz(v->l) + sz(v->r) + 1;

    v->sum = sum(v->l) + sum(v->r) + v->val * v->cnt;
    v->cnt = cnt(v->l) + cnt(v->r) + v->inds.size();
}

void push(node *v) {
    if (!v) return;
    upd(v);
    v->val += v->st + sz(v->l) * v->del;
    v->sum += (v->st + (sz(v)-1) * v->del) * v->sz / 2;
    if (v->l) {
        v->l->st += v->st;
        v->l->del += v->del;
    }
    if (v->r) {
        v->r->st += v->st + (v->del * (sz(v->l) + 1));
        v->r->del += v->del;
    }
    v->st = 0;
    v->del = 0;
}


int dfs_sum(node *v) {
    if (!v) return 0;
    push(v);
    return v->val * v->inds.size() + dfs_sum(v->l) + dfs_sum(v->r);
}

pair<node*, node*> split(node *t, int k) {
    if (!t) return {nullptr, nullptr};
    push(t);
    if (sz(t->l) >= k) {
        if (t->l) t->l->pr = nullptr;
        auto [l, r] = split(t->l, k);
        t->l = r;
        if (r) r->pr = t;
        upd(t);
        return {l, t};
    } else {
        if (t->r) t->r->pr = nullptr;
        auto [l, r] = split(t->r, k - sz(t->l) - 1);
        t->r = l;
        if (l) l->pr = t;
        upd(t);
        return {t, r};
    }
}

pair<node*, node*> split2(node *t, int x) {
    if (!t) return {nullptr, nullptr};
    push(t);
    if (t->val >= x) {
        if (t->l) t->l->pr = nullptr;
        auto [l, r] = split2(t->l, x);
        t->l = r;
        if (r) r->pr = t;
        upd(t);
        return {l, t};
    } else {
        if (t->r) t->r->pr = nullptr;
        auto [l, r] = split2(t->r, x);
        t->r = l;
        if (l) l->pr = t;
        upd(t);
        return {t, r};
    }
}

node* merge(node *L, node *R) {
    push(L); push(R);
    if (!L) return R;
    if (!R) return L;
    if (L->y < R->y) {
        L->r = merge(L->r, R);
        L->r->pr = L;
        upd(L);
        return L;
    } else {
        R->l = merge(L, R->l);
        R->l->pr = R;
        upd(R);
        return R;
    }
}

node *next(node *v) {
    return (v->next == nullptr ? v : v->next = next(v->next));
}

node* add_seg(node *v, int st, int del) {
    assert(v != nullptr);
    v->st += st;
    v->del += del;
    push(v);
    return v;
}

node *mx;
void dfs(node *v) {
    if (mx->inds.size() < v->inds.size()) {
        swap(mx->inds, v->inds);
    }
    if (v->l) dfs(v->l);
    if (v->r) dfs(v->r);
}

vector<node*> ind;
void dfs2(node *v, node *st) {
    if (v != mx) {
        for (int i : v->inds) {
            st->inds.push_back(i);
            ind[i] = st;
        }
    }
    if (v != st) {
        v->next = st;
    }
    if (v->l) dfs2(v->l, st);
    if (v->r) dfs2(v->r, st);
}

int32_t main() {
    ios_base::sync_with_stdio(0);
    cin.tie(0);

    int n, q;
    cin >> n >> q;
    vector<int> mas(n);
    for (auto &i : mas) {
        cin >> i;
    }
    vector<vector<int>> que;
    for (int i = 0; i < q; ++i) {
        int tp, k, g = 0;
        cin >> tp >> k;
        if (tp == 3) {
            cin >> g;
            g--;
        }
        que.push_back({k, tp, g, i});
    }
    int SS = 0;
    sort(all(que));
    vector<int> ans(q);
    map<int, vector<int>> cnt;
    for (int i = 0; i < n; ++i) {
        SS += mas[i];
        cnt[mas[i]].push_back(i);
    }
    set<pair<int, pair<node*, node*>>> pr;
    ind.resize(n);
    node *root = nullptr;
    node *last = nullptr;
    for (auto i : cnt) {
        node *nw = new node();
        nw->val = i.fi;
        nw->inds = i.se;
        nw->cnt = i.se.size();
        for (int j : i.se) {
            ind[j] = nw;
        }
        if (last != nullptr) {
            pr.insert({nw->val - last->val, {last, nw}});
        }
        last = nw;
        root = merge(root, nw);
    }

    int sz = cnt.size();
    int cur_t = 0;
    for (int Q = 0; Q < q; ++Q) {
        auto ev = que[Q];
        int k = ev[0], tp = ev[1], g = ev[2];
        while (!pr.empty() && pr.begin()->fi <= (k)) {

            root = add_seg(root, (sz-1) * (pr.begin()->fi-cur_t), -(pr.begin()->fi-cur_t));

            vector<node*> sus;
            for (node* i = next(pr.begin()->se.fi); i != nullptr; i = i->pr) {
                sus.push_back(i);
            }
            reverse(all(sus));
            for (auto i : sus) {
                push(i);
            }
            int val = next(pr.begin()->se.fi)->val;
            auto [l1, r1] = split2(root, val);
            auto [l2, r2] = split2(r1, val+1);

            mx = l2;
            dfs(l2);
            if (mx != l2) {
                swap(l2->inds, mx->inds);
            }
            dfs2(l2, l2);
            l2->l = l2->r = nullptr;
            l2->cnt = l2->inds.size();
            upd(l2);

            root = merge(l1, merge(l2, r2));

            cur_t = pr.begin()->fi;
            sz--;
            pr.erase(pr.begin());
        }
        root = add_seg(root, (sz-1) * (k - cur_t), -(k - cur_t));
        cur_t = k;
        if (tp == 1) {
            assert(sz == root->sz);
            ans[ev[3]] = sz;
        } else if (tp == 2) {
            ans[ev[3]] = dfs_sum(root) - SS;
        } else {
            vector<node*> sus;
            for (node* i = next(ind[g]); i != nullptr; i = i->pr) {
                sus.push_back(i);
            }
            reverse(all(sus));
            for (auto i : sus) {
                push(i);
            }
            ans[ev[3]] = ind[g]->val;
        }
    }
    for (int i : ans) {
        cout << i << '\n';
    }
}